from functools import partial
from numpy import busday_count as original_busday_count, datetime64, timedelta64, where
from openfisca_france.model.base import *


class indemnites_stage(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités de stage'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class revenus_stage_formation_pro(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus de stage de formation professionnelle'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class bourse_recherche(Variable):
    value_type = float
    entity = Individu
    label = 'Bourse de recherche'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class sal_pen_exo_etr(Variable):
    cerfa_field = {
        0: '1AC',
        1: '1BC',
        2: '1CC',
        3: '1DC',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Salaires et pensions exonérés de source étrangère retenus pour le calcul du taux effectif'
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class frais_reels(Variable):
    cerfa_field = {
        0: '1AK',
        1: '1BK',
        2: '1CK',
        3: '1DK',
        4: '1EK',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Frais réels'
    definition_period = YEAR


class hsup(Variable):
    cerfa_field = {
        0: '1AU',
        1: '1BU',
        2: '1CU',
        3: '1DU',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Heures supplémentaires : revenus exonérés connus'
    # start_date = date(2007, 1, 1)
    end = '2013-12-13'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add


class ppe_du_sa(Variable):
    cerfa_field = {
        0: '1AV',
        1: '1BV',
        2: '1CV',
        3: '1DV',
        4: '1QV',
        }
    value_type = int
    entity = Individu
    label = "Prime pour l'emploi des salariés: nombre d'heures payées"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        heures_remunerees_volume = individu('heures_remunerees_volume', period)
        contrat_travail = individu('contrat_de_travail', period)
        travail_temps_decompte_en_heures = (
            (contrat_travail == TypesContratDeTravail.temps_partiel)
            + (contrat_travail == TypesContratDeTravail.forfait_heures_semaines)
            + (contrat_travail == TypesContratDeTravail.forfait_heures_mois)
            + (contrat_travail == TypesContratDeTravail.forfait_heures_annee)
            + (contrat_travail == TypesContratDeTravail.forfait_jours_annee)
            )

        return heures_remunerees_volume * travail_temps_decompte_en_heures


class ppe_tp_sa(Variable):
    cerfa_field = {
        0: '1AX',
        1: '1BX',
        2: '1CX',
        3: '1DX',
        4: '1QX',
        }
    value_type = bool
    entity = Individu
    label = "Prime pour l'emploi des salariés: indicateur de travail à temps plein sur l'année entière"
    definition_period = YEAR

    def formula(individu, period):
        mois = period.first_month
        indicateur = individu('contrat_de_travail', mois) == 0
        # On parcours tous les mois de l'année pour s'assurer que l'individu était employé à temps plein
        # durant toute l'année.
        while mois.start.month < 12:
            mois = mois.offset(1)
            indicateur = indicateur & (individu('contrat_de_travail', mois) == 0)
        return indicateur


class TypesExpositionAccident(Enum):
    __order__ = 'faible moyen eleve tres_eleve'  # Needed to preserve the enum order in Python 2
    faible = 'Faible'
    moyen = 'Moyen'
    eleve = 'Élevé'
    tres_eleve = 'Très élevé'


class exposition_accident(Variable):
    value_type = Enum
    possible_values = TypesExpositionAccident
    default_value = TypesExpositionAccident.faible
    entity = Individu
    label = 'Exposition au risque pour les accidents du travail'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesExpositionPenibilite(Enum):
    __order__ = 'nulle simple multiple'  # Needed to preserve the enum order in Python 2
    nulle = "Nulle, pas d'exposition de l'employé à un facteur de pénibilité"
    simple = 'Simple, exposition à un seul facteur de pénibilité'
    multiple = 'Multiple, exposition à plusieurs facteurs de pénibilité'


class exposition_penibilite(Variable):
    value_type = Enum
    possible_values = TypesExpositionPenibilite
    default_value = TypesExpositionPenibilite.nulle
    entity = Individu
    label = 'Exposition à un ou plusieurs facteurs de pénibilité'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesAllegementModeRecouvrement(Enum):
    # Informations sur la régularisation de la réduction appliquée sur le site des URSSAF : https://www.urssaf.fr/portail/home/employeur/beneficier-dune-exoneration/exonerations-generales/la-reduction-generale/le-calcul-de-la-reduction/etape-2--le-calcul-de-la-reducti/la-regularisation.html
    __order__ = 'fin_d_annee anticipe progressif'  # Needed to preserve the enum order in Python 2
    fin_d_annee = "Paiement en fin d'année des cotisations avec l'allègement exact"
    anticipe = "Paiement anticipé des cotisations et régularisation de l'allègement en fin de période"
    progressif = "Paiement anticipé des cotisations et régularisation progressive de l'allègement"  # La régularisation est faite à chaque paiement anticipé, «en faisant masse des éléments nécessaires au calcul de la réduction»


class allegement_fillon_mode_recouvrement(Variable):
    value_type = Enum
    possible_values = TypesAllegementModeRecouvrement
    default_value = TypesAllegementModeRecouvrement.fin_d_annee
    entity = Individu
    label = 'Mode de recouvrement des allègements Fillon'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class allegement_cotisation_allocations_familiales_mode_recouvrement(Variable):
    value_type = Enum
    possible_values = TypesAllegementModeRecouvrement
    default_value = TypesAllegementModeRecouvrement.fin_d_annee
    entity = Individu
    label = "Mode de recouvrement de l'allègement de la cotisation d'allocations familiales"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class allegement_cotisation_maladie_mode_recouvrement(Variable):
    value_type = Enum
    possible_values = TypesAllegementModeRecouvrement
    default_value = TypesAllegementModeRecouvrement.fin_d_annee
    entity = Individu
    label = "Mode de recouvrement de l'allègement des cotisations maladie sur les bas et moyens salaires (Ex-CICE)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class apprentissage_contrat_debut(Variable):
    value_type = date
    entity = Individu
    label = "Date de début du contrat d'apprentissage"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class arrco_tranche_a_taux_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Taux ARRCO tranche A employeur) propre à l'entreprise"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class arrco_tranche_a_taux_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Taux ARRCO tranche A salarié) propre à l'entreprise"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class assujettie_taxe_salaires(Variable):
    value_type = bool
    entity = Individu
    label = 'Entreprise assujettie à la taxe sur les salaires'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class avantage_en_nature_valeur_reelle(Variable):
    value_type = float
    entity = Individu
    label = 'Avantages en nature (Valeur réelle)'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_compensatrices_conges_payes(Variable):
    value_type = float
    entity = Individu
    label = 'indemnites_compensatrices_conges_payes'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnite_fin_contrat_due(Variable):
    value_type = bool
    entity = Individu
    label = 'indemnite_fin_contrat_due'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class TypesContratDeTravail(Enum):
    __order__ = 'temps_plein temps_partiel forfait_heures_semaines forfait_heures_mois forfait_heures_annee forfait_jours_annee sans_objet'  # Needed to preserve the enum order in Python 2
    temps_plein = 'Temps plein'
    temps_partiel = 'Temps partiel'
    forfait_heures_semaines = 'Convention de forfait heures sur la semaine'
    forfait_heures_mois = 'Convention de forfait heures sur le mois'
    forfait_heures_annee = 'Convention de forfait heures sur l’année'
    forfait_jours_annee = 'Convention de forfait hours sur l’année'
    sans_objet = 'Non renseigné'


class contrat_de_travail(Variable):
    value_type = Enum
    possible_values = TypesContratDeTravail
    default_value = TypesContratDeTravail.temps_plein
    entity = Individu
    label = 'Type de contrat de travail'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class date_debut_recherche_emploi(Variable):
    value_type = date
    default_value = date(1, 1, 1)
    entity = Individu
    label = "Date de début d'une activité en recherche d'emploi selon Pôle Emploi (exemple : entretien d'embauche, concours public, examen certifiant)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class contrat_de_travail_debut(Variable):
    value_type = date
    default_value = date(1870, 1, 1)
    entity = Individu
    label = "Date d'arrivée dans l'entreprise"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class contrat_de_travail_fin(Variable):
    value_type = date
    default_value = date(2099, 12, 31)
    entity = Individu
    label = "Date de départ de l'entreprise"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesContrat(Enum):
    __order__ = 'aucun cdi cdd ctt formation'  # Needed to preserve the enum order in Python 2
    aucun = 'Aucun contrat'
    cdi = 'Contrat de travail à durée indéterminée (CDI)'
    cdd = 'Contrat de travail à durée déterminée (CDD)'
    ctt = 'Contrat de travail temporaire (CTT)'
    formation = 'Formation'


class contrat_de_travail_type(Variable):
    value_type = Enum
    possible_values = TypesContrat
    default_value = TypesContrat.cdi
    entity = Individu
    label = 'Type du contrat de travail'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesContratTravailDureeDeterminee(Enum):
    '''
    Tous types contrats : https://travail-emploi.gouv.fr/droit-du-travail/les-contrats-de-travail/
    CDD, types et durées : https://www.service-public.fr/particuliers/vosdroits/F36

    Contrat vendanges : https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000006585176/
    CDD d'insertion : https://www.service-public.fr/particuliers/vosdroits/F14100

    Contrat unique d'insertion (CUI), types : https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000019869576
    PEC en CDD ou CDI : https://travail-emploi.gouv.fr/emploi-et-insertion/parcours-emploi-competences/pec

    Articulation CUI/PEC et secteur : https://www.service-public.fr/particuliers/vosdroits/F21006
    CUI-CIE = Parcours emploi compétence (PEC) - secteur marchand
    CUI-CAE = Parcours emploi compétence (PEC) - secteur non marchand
    '''
    __order__ = 'non_renseigne contrat_general contrat_saisonnier contrat_vendanges contrat_usage contrat_insertion contrat_initiative_emploi contrat_accompagnement_emploi'  # Needed to preserve the enum order in Python 2

    non_renseigne = 'Non renseigné'
    contrat_general = 'Cas général du contrat de travail à durée déterminée (CDD)'

    contrat_saisonnier = 'CDD à caractère saisonnier'
    contrat_vendanges = 'CDD à caractère saisonnier pour travaux de vendanges'
    contrat_usage = "CDD d'usage (CDDU)"  # exemple : CDD d'extra
    contrat_insertion = "CDD d'insertion (CDDI)"

    contrat_initiative_emploi = "Contrat unique d'insertion - Contrat initiative emploi (CUI-CIE)"
    contrat_accompagnement_emploi = "Contrat unique d'insertion - Contrat d'accompagnement dans l'emploi (CUI-CAE)"


class contrat_duree_determinee_type(Variable):
    value_type = Enum
    possible_values = TypesContratTravailDureeDeterminee
    default_value = TypesContratTravailDureeDeterminee.non_renseigne
    entity = Individu
    label = 'Type du contrat de travail à durée déterminée (CDD)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class contrat_aide(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est en contrat aidé"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://dares.travail-emploi.gouv.fr/definitions-et-concepts/contrats-aides'


class duree_formation(Variable):
    value_type = float
    entity = Individu
    label = 'Durée de la formation en heures'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class TypesLieuEmploiFormation(Enum):
    non_renseigne = 'Non renseigné'
    metropole_hors_corse = 'Métropole hors Corse'
    corse = 'Corse'
    guadeloupe = 'Guadeloupe'
    martinique = 'Martinique'
    guyane = 'Guyane'
    la_reunion = 'La réunion'
    saint_pierre_et_miquelon = 'Saint Pierre et Miquelon'
    mayotte = 'Mayotte'
    saint_bartelemy = 'Saint Bartelemy'
    saint_martin = 'Saint Martin'


class lieu_emploi_ou_formation(Variable):
    value_type = Enum
    possible_values = TypesLieuEmploiFormation
    default_value = TypesLieuEmploiFormation.non_renseigne
    entity = Individu
    label = "Zone de l'emploi ou de la formation"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class contrat_de_travail_duree(Variable):
    value_type = float
    default_value = 0.
    entity = Individu
    label = 'Durée du contrat de travail en mois'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesCategoriesDemandeurEmploi(Enum):
    # http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-n2016-33-du-6-octobre-2016-bope-n2016-80.html?type=dossiers/2016/bope-n2016-80-du-17-novembre-201
    # https://allocation-chomage.fr/categorie-chomeur/

    __order__ = 'pas_de_categorie categorie_1 categorie_2 categorie_3 categorie_4 categorie_5 categorie_6 categorie_7 categorie_8'\
                # Needed to preserve the enum order in Python 2

    pas_de_categorie = 'Aucune catégorie'
    categorie_1 = 'Catégorie 1 - Personnes sans emploi, immédiatement disponibles en recherche de CDI plein temps.'
    categorie_2 = 'Catégorie 2 - Personnes sans emploi, immédiatement disponibles en recherche de CDI à temps partiel.'
    categorie_3 = 'Catégorie 3 - Personnes sans emploi, immédiatement disponibles en recherche de CDD.'
    categorie_4 = 'Catégorie 4 - Personnes sans emploi, non immédiatement disponibles et à la recherche d’un emploi.'
    categorie_5 = "Catégorie 5 - Personnes non immédiatement disponibles, parce que titulaires d'un ou de plusieurs emplois, et à la recherche d'un autre emploi."
    categorie_6 = "Catégorie 6 - Personnes non immédiatement disponibles, en recherche d'un autre emploi en CDI à plein temps."
    categorie_7 = "Catégorie 7 - Personnes non immédiatement disponibles, en recherche d'un autre emploi en CDI à temps partiel."
    categorie_8 = "Catégorie 8 - Personnes non immédiatement disponibles, en recherche d'un autre emploi en CDD."


class pole_emploi_categorie_demandeur_emploi(Variable):
    reference = [
        'http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-n2016-33-du-6-octobre-2016-bope-n2016-80.html?type=dossiers/2016/bope-n2016-80-du-17-novembre-201',
        'Annexe 3 : la fiche 3 - Les effets de l’inscription'
        ]
    value_type = Enum
    possible_values = TypesCategoriesDemandeurEmploi
    default_value = TypesCategoriesDemandeurEmploi.pas_de_categorie
    entity = Individu
    label = 'Le classement des demandeurs d’emploi dans les différentes catégories d’inscription à Pôle Emploi'
    definition_period = MONTH


class TypesCotisationSocialeModeRecouvrement(Enum):
    __order__ = 'mensuel annuel mensuel_strict'  # Needed to preserve the enum order in Python 2
    mensuel = "Mensuel avec régularisation en fin d'année"
    annuel = 'Annuel'
    mensuel_strict = 'Mensuel strict'


class cotisation_sociale_mode_recouvrement(Variable):
    value_type = Enum
    possible_values = TypesCotisationSocialeModeRecouvrement
    default_value = TypesCotisationSocialeModeRecouvrement.mensuel_strict
    entity = Individu
    label = 'Mode de recouvrement des cotisations sociales'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class entreprise_est_association_non_lucrative(Variable):
    value_type = bool
    entity = Individu
    label = "L'entreprise est une association à but non lucratif, par exemple loi de 1901"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class depcom_entreprise(Variable):
    value_type = str
    max_length = 5
    entity = Individu
    label = 'Localisation entreprise (depcom)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class code_postal_entreprise(Variable):
    value_type = str
    max_length = 5
    entity = Individu
    label = 'Localisation entreprise (Code postal)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class salarie_regime_alsace_moselle(Variable):
    entity = Individu
    value_type = bool
    label = "Le salarié cotise au régime de l'Alsace-Moselle"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    # Attention : ce n'est pas équivalent au fait de travailler en Alsace-Moselle !
    # http://regime-local.fr/salaries/


class effectif_entreprise(Variable):
    entity = Individu
    value_type = int
    label = "Effectif de l'entreprise"
    set_input = set_input_dispatch_by_period
    definition_period = MONTH


class entreprise_assujettie_cet(Variable):
    value_type = bool
    entity = Individu
    label = 'Entreprise assujettie à la contribution économique territoriale'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class entreprise_assujettie_is(Variable):
    value_type = bool
    entity = Individu
    label = "Entreprise assujettie à l'impôt sur les sociétés (IS)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class entreprise_benefice(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Bénéfice de l'entreprise"
    definition_period = MONTH
    calculate_output = calculate_output_add


class entreprise_bilan(Variable):
    value_type = float
    entity = Individu
    label = "Bilan de l'entreprise"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class entreprise_chiffre_affaire(Variable):
    value_type = float
    entity = Individu
    label = "Chiffre d'affaire de l'entreprise"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class entreprise_creation(Variable):
    value_type = date
    entity = Individu
    label = "Date de création de l'entreprise"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class nombre_tickets_restaurant(Variable):
    value_type = int
    entity = Individu
    label = 'Nombre de tickets restaurant'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class nouvelle_bonification_indiciaire(Variable):
    value_type = float
    entity = Individu
    label = 'Nouvelle bonification indicaire'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class prevoyance_obligatoire_cadre_taux_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Taux de cotisation employeur pour la prévoyance obligatoire des cadres'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class primes_salaires(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités, primes et avantages en argent (brut)'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class prime_partage_valeur(Variable):
    value_type = float
    entity = Individu
    label = 'Prime pérenne de partage de la valeur (PPV)'
    definition_period = (YEAR)  # La PPV est versée en fonction du salaire des 12 derniers mois
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000046188457/2022-08-18'
    set_input = set_input_divide_by_period
    documentation = '''
        La PPV exonérée représente l'éxonération de la prime des cotisations salariales,
        patronales et l'impôt sur le revenu. La PPV prévoit l'absence de substitution et
        donc le caractère « fantôme » de la prime au regard des ressources des
        administrations publiques et singulièrement de la sécurité sociale.

        la condition de rémunération est valable jusqu'au 31 décembre 2023.
        Alors, lorsque la rémunération est inférieure à 3 SMIC, la PPV est **aussi**
        exonérée d'impôt sur le revenu, ainsi que des contributions prévues
        à l'article L. 136-1 du code de la sécurité sociale
        [CSG activité = https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000033712581]
        et à l'article 14 de l'ordonnance n° 96-50 du 24 janvier 1996 relative au remboursement
        de la dette sociale (⑯).
        [CRDS = https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000038834962/]
        Néanmoins, elle est incluse dans le revenu fiscal de référence (⑰).
        => Sous 3 SMIC les 12 derniers mois, on est en plus exonéré d'IR, CSG et CRDS.

        Pour tout niveau de revenu :
        PPV est exonérée, dans la limite de 3 000 euros :
        * de toutes les cotisations sociales d'origine légale ou conventionnelle à la charge du salarié et de l'employeur,
        ainsi que des participations, taxes et contributions prévues :
        * à l'article 235 bis du code général des impôts
        [PEEC applicable au-dessus de 50 salariés = https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000038586341/]
        * et à l'article L. 6131-1 du code du travail,
        [
            contribution unique à la formation professionnelle (CFP ?),
            la contribution supplémentaire,
            contribution dédiée au financement du compte personnel de formation
            = article L. 6131-1 du code du travail
            ]
        dans leur rédaction en vigueur à la date de son versement.
        '''


class prime_partage_valeur_exceptionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Prime exceptionnelle de partage de la valeur (PPV)'
    definition_period = (YEAR)  # La PPV est versée en fonction du salaire des 12 derniers mois
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000046188457/2022-08-18'
    set_input = set_input_divide_by_period


class prime_partage_valeur_exoneree_exceptionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Prime exceptionnelle de partage de la valeur (PPV), partie exonérée'
    definition_period = YEAR
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000046188457/2022-08-18'

    set_input = set_input_divide_by_period

    def formula_2022_07_01(individu, period, parameters):
        '''
        La prime exceptionnelle de partage de la valeur (PPV),
        est réservée aux salariés qui ont un salaire de base inférieur à 3 x SMIC.
        Elle ne peut plus être versée après le 31 décembre 2023.
        '''

        prime_partage_valeur = individu('prime_partage_valeur_exceptionnelle', period)
        accord_interessement = individu('accord_interessement', period.first_month)
        ppv_parameters = parameters(period).marche_travail.primes_exceptionnelles.prime_partage_valeur
        plafond_ppv_exoneree = where(
            accord_interessement,
            ppv_parameters.plafond_exoneration_avec_accord_interessement,
            ppv_parameters.plafond_exoneration,
            )
        # Le plafond doit être diminué de la prime PEPA éventuellement versée début 2022
        prime_exceptionnelle_pouvoir_achat = individu('prime_exceptionnelle_pouvoir_achat', period)
        plafond_ppv_exoneree = plafond_ppv_exoneree - prime_exceptionnelle_pouvoir_achat
        ppv_eligibilite_exceptionnelle = individu('ppv_eligibilite_exceptionnelle', period)
        return (
            min_(prime_partage_valeur, plafond_ppv_exoneree)
            * ppv_eligibilite_exceptionnelle  # Neutralisation de la prime pour >= 3 x SMIC
            )


class prime_partage_valeur_non_exoneree_exceptionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Prime exceptionnelle de partage de la valeur (PPV), partie non exonérée'
    definition_period = YEAR
    set_input = set_input_divide_by_period

    def formula_2022_07_01(individu, period, parameters):
        prime_partage_valeur_exceptionnelle = individu('prime_partage_valeur_exceptionnelle', period)
        ppv_eligibilite_exceptionnelle = individu('ppv_eligibilite_exceptionnelle', period)
        prime_partage_valeur_exoneree_exceptionnelle = individu('prime_partage_valeur_exoneree_exceptionnelle', period)
        return (
            prime_partage_valeur_exceptionnelle
            - prime_partage_valeur_exoneree_exceptionnelle
            ) * ppv_eligibilite_exceptionnelle  # Neutralisation de la prime pour >= 3 x SMIC


class ppv_eligibilite_exceptionnelle(Variable):
    '''
    Cette variable sert à neutraliser la prime pour les personnes qui touchent plus que 3xSMIC
    Car dans ce cas il n'est pas autorisé de leur verser la prime exceptionnelle.
    L'employeur doit alors opter pour la prime temporaire/exceptionnelle.
    '''

    value_type = float
    entity = Individu
    label = 'Eligibilité aux exonérations complémentaires pour la PPV'
    definition_period = YEAR
    set_input = set_input_dispatch_by_period
    end = '2023-12-31'
    documentation = '''
    L'individu est éligible à des exonérations complémentaires
    sur la prime de partage de valeur (PPV) pour une rémunération
    inférieure à 3 SMIC : exonération de CSG, CRDS
    et impôt sur le revenu.
    '''

    def formula_2022_07_01(individu, period, parameters):
        annee_glissante = period.start.period('year').offset(-1)
        salaire_de_base_annuel = individu('salaire_de_base', annee_glissante, options=[ADD])
        smic_b_annuel = parameters(period).marche_travail.salaire_minimum.smic.smic_b_mensuel * 12
        quotite_de_travail = individu('quotite_de_travail', period, options=[ADD]) / 12
        plafond_salaire = parameters(period).marche_travail.primes_exceptionnelles.prime_partage_valeur.plafond_salaire
        return (salaire_de_base_annuel) < (
            smic_b_annuel * plafond_salaire * quotite_de_travail
            )


class prime_partage_valeur_exoneree(Variable):
    value_type = float
    entity = Individu
    label = 'Prime pérenne de partage de la valeur (PPV), partie exonérée'
    definition_period = YEAR
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000046188457/2022-08-18'
    set_input = set_input_divide_by_period

    def formula_2022_07_01(individu, period, parameters):
        '''
        Il y a deux plafond suivant que l'employeur ait ou non :
        # * un dispositif d'intéressement,
        # * TODO : par un organisme d'intérêt général
        # * TODO : ou, s'agissant des primes versées aux travailleurs handicapés,
        #          par un établissement ou service d'aide par le travail
        '''

        prime_partage_valeur = individu('prime_partage_valeur', period)
        accord_interessement = individu('accord_interessement', period.first_month)

        ppv_parameters = parameters(period).marche_travail.primes_exceptionnelles.prime_partage_valeur
        plafond_ppv_exoneree = where(
            accord_interessement,
            ppv_parameters.plafond_exoneration_avec_accord_interessement,
            ppv_parameters.plafond_exoneration,
            )
        return min_(prime_partage_valeur, plafond_ppv_exoneree)


class prime_partage_valeur_non_exoneree(Variable):
    value_type = float
    entity = Individu
    label = 'Prime pérenne de partage de la valeur (PPV), partie non exonérée'
    definition_period = YEAR
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000046188457/2022-08-18'
    set_input = set_input_divide_by_period

    def formula_2022_07_01(individu, period, parameters):
        prime_partage_valeur = individu('prime_partage_valeur', period)
        prime_partage_valeur_exoneree = individu(
            'prime_partage_valeur_exoneree', period
            )
        return prime_partage_valeur - prime_partage_valeur_exoneree


class prime_exceptionnelle_pouvoir_achat(Variable):
    value_type = float
    entity = Individu
    label = "Prime exceptionnelle de pouvoir d'achat (prime Macron)"
    definition_period = YEAR  # La pepa est versée en fonction du salaire des 12 derniers mois
    set_input = set_input_divide_by_period


class accord_interessement(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu travaille dans une entreprise couverte par un accord d'intéressement"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class prime_exceptionnelle_pouvoir_achat_exoneree(Variable):
    value_type = float
    entity = Individu
    label = "Prime exceptionnelle de pouvoir d'achat (prime Macron), partie exonérée"
    definition_period = YEAR
    reference = 'https://www.legifrance.gouv.fr/jorf/article_jo/JORFARTI000043805912'
    #  La pepa exonérée représente l'éxonération de la prime des cotisations salariales, patronales et l’impôt sur le revenu. La pepa prévoit l'absence de substitution et donc le caractère « fantôme » de la prime au regard des ressources des administrations publiques et singulièrement de la sécurité sociale.
    set_input = set_input_divide_by_period
    end = '2022-03-31'

    def formula_2019_01_01(individu, period, parameters):
        '''
        Voici l'explication du dispositif :
        https://boss.gouv.fr/portail/accueil/mesures-exceptionnelles/instruction-du-19-aout-2021.html
        Prime exceptionnelle de pouvoir d'achat (prime Macron), voici comment j'ai écrit mon calcul :
        si salaire < 3 SMIC alors
            si prime pepa < 1000 alors
                exoneration = min (prime pepa, 1000)
            sinon (cas prime pepa >= 1000)
                si accord interessement ou effectif salarié < 50
                    alors exoneration= min (prime pepa,2000)
                sinon
                    exoneration = min (prime pepa, 1000)
        sinon
            Pas d'exonération
        '''
        annee_glissante = period.start.period('year').offset(-1)
        salaire_de_base_annuel = individu('salaire_de_base', annee_glissante, options=[ADD])
        smic_b_annuel = parameters(period).marche_travail.salaire_minimum.smic.smic_b_mensuel * 12
        quotite_de_travail = individu('quotite_de_travail', period, options=[ADD]) / 12
        plafond_salaire = parameters(period).marche_travail.primes_exceptionnelles.prime_pepa.plafond_salaire

        # "une rémunération inférieure à trois fois la valeur annuelle du salaire minimum de croissance
        # correspondant à la durée de travail prévue au contrat"
        condition_remuneration = (
            salaire_de_base_annuel
            ) < (
                smic_b_annuel * plafond_salaire * quotite_de_travail
                )

        prime_exceptionnelle_pouvoir_achat = individu(
            'prime_exceptionnelle_pouvoir_achat',
            period)

        plafond_exoneration = parameters(period).marche_travail.primes_exceptionnelles.prime_pepa.plafond_exoneration
        prime_inf_seuil_1 = prime_exceptionnelle_pouvoir_achat <= plafond_exoneration

        accord_interessement = individu('accord_interessement', period.first_month)
        effectif_entreprise = individu('effectif_entreprise', period.first_month)
        plafond_effectif_entreprise = parameters(period).marche_travail.primes_exceptionnelles.prime_pepa.plafond_effectif_entreprise
        condition_entreprise = accord_interessement + (effectif_entreprise < plafond_effectif_entreprise)
        plafond_exoneration_avec_accord_interessement = parameters(period).marche_travail.primes_exceptionnelles.prime_pepa.plafond_exoneration_avec_accord_interessement
        return (condition_remuneration
                * where(
                    prime_inf_seuil_1,
                    min_(prime_exceptionnelle_pouvoir_achat, plafond_exoneration),
                    where(
                        condition_entreprise,
                        min_(prime_exceptionnelle_pouvoir_achat, plafond_exoneration_avec_accord_interessement),
                        plafond_exoneration
                        )
                    )
                )


class prime_exceptionnelle_pouvoir_achat_non_exoneree(Variable):
    value_type = float
    entity = Individu
    label = "Prime exceptionnelle de pouvoir d'achat (prime Macron), partie non exonérée"
    definition_period = YEAR
    set_input = set_input_divide_by_period
    end = '2022-03-31'

    def formula_2019_01_01(individu, period, parameters):
        prime_exceptionnelle_pouvoir_achat = individu('prime_exceptionnelle_pouvoir_achat', period)
        prime_exceptionnelle_pouvoir_achat_exoneree =\
            individu('prime_exceptionnelle_pouvoir_achat_exoneree', period)
        return prime_exceptionnelle_pouvoir_achat - prime_exceptionnelle_pouvoir_achat_exoneree


class primes_salaires_non_exonerees(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités, primes et avantages en argent non exonérés (brut)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2022_07_01(individu, period, parameters):
        primes_salaires = individu('primes_salaires', period)
        prime_partage_valeur_non_exoneree = individu('prime_partage_valeur_non_exoneree', period, options=[DIVIDE])
        prime_partage_valeur_non_exoneree_exceptionnelle = individu('prime_partage_valeur_non_exoneree_exceptionnelle', period, options=[DIVIDE])
        return (
            primes_salaires
            + prime_partage_valeur_non_exoneree
            + prime_partage_valeur_non_exoneree_exceptionnelle
            )

    def formula_2019_01_01(individu, period, parameters):
        primes_salaires = individu('primes_salaires', period)
        prime_exceptionnelle_pouvoir_achat_non_exoneree =\
            individu('prime_exceptionnelle_pouvoir_achat_non_exoneree', period, options = [DIVIDE])
        return primes_salaires + prime_exceptionnelle_pouvoir_achat_non_exoneree

    def formula(individu, period, parameters):
        primes_salaires = individu('primes_salaires', period)
        return primes_salaires


class complementaire_sante_montant(Variable):
    value_type = float
    entity = Individu
    label = "Montant de la complémentaire santé obligatoire retenue par l'employeur"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class complementaire_sante_part_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Part de la complémentaire santé obligatoire payée par l'employeur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class prevoyance_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Contributions de prévoyance prises en charge par l'employeur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class prise_en_charge_employeur_retraite_complementaire(Variable):
    value_type = float
    entity = Individu
    label = "Part des cotisations de retraite complémentaire prise en charge par l'employeur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class prise_en_charge_employeur_retraite_supplementaire(Variable):
    value_type = float
    entity = Individu
    label = "Part salariale des cotisations de retraite supplémentaire prise en charge par l'employeur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class ratio_alternants(Variable):
    value_type = float
    entity = Individu
    label = "Ratio d'alternants dans l'effectif moyen"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    # TODO modéliser cette variable, dont la définition intervient dans le calcul de la Contribution supplémentaire d'apprentissage, et change au cours du temps.


class remboursement_transport_base(Variable):
    value_type = float
    entity = Individu
    label = 'Base pour le calcul du remboursement des frais de transport'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_forfaitaires(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités forfaitaires (transport, nourriture)'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class salaire_de_base(Variable):
    # Salaire brut sans les primes et les heures supplémentaires - généralement la première ligne du bulletin de paye.
    value_type = float
    entity = Individu
    label = 'Salaire de base'
    set_input = set_input_divide_by_period
    reference = 'https://www.insee.fr/fr/metadonnees/definition/c1937'
    definition_period = MONTH
    unit = 'currency'


class titre_restaurant_taux_employeur(Variable):
    value_type = float
    default_value = 0.5
    entity = Individu
    label = "Taux de participation de l'employeur au titre restaurant"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class titre_restaurant_valeur_unitaire(Variable):
    value_type = float
    entity = Individu
    label = 'Valeur faciale unitaire du titre restaurant'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class titre_restaurant_volume(Variable):
    value_type = int
    entity = Individu
    label = 'Volume des titres restaurant'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class traitement_indiciaire_brut(Variable):
    value_type = float
    entity = Individu
    label = 'Traitement indiciaire brut (TIB)'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class categorie_salarie(Variable):
    value_type = Enum
    possible_values = TypesCategorieSalarie  # defined in model/base.py
    default_value = TypesCategorieSalarie.prive_non_cadre
    entity = Individu
    label = 'Catégorie de salarié'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class heures_duree_collective_entreprise(Variable):
    value_type = int  # TODO default la valeur de la durée légale ?
    entity = Individu
    label = "Durée mensuelle collective dans l'entreprise (heures, temps plein)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class heures_non_remunerees_volume(Variable):
    value_type = float
    entity = Individu
    label = 'Volume des heures non rémunérées (convenance personnelle hors contrat/forfait)'
    set_input = set_input_divide_by_period
    definition_period = MONTH


class heures_remunerees_volume(Variable):
    # N'est pas pris en compte lorsque type_contrat_travail = temps_plein
    value_type = float
    entity = Individu
    label = 'Volume des heures rémunérées contractuellement'
    set_input = set_input_divide_by_period
    definition_period = MONTH


class forfait_heures_remunerees_volume(Variable):
    value_type = int
    entity = Individu
    label = 'Volume des heures rémunérées à un forfait heures'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class forfait_jours_remuneres_volume(Variable):
    value_type = int
    entity = Individu
    label = 'Volume des heures rémunérées à forfait jours'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class volume_jours_ijss(Variable):
    value_type = int
    entity = Individu
    label = 'Volume des jours pour lesquels sont versés une idemnité journalière par la sécurité sociale'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class avantage_en_nature(Variable):
    value_type = float
    entity = Individu
    label = 'Avantages en nature'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        period = period
        avantage_en_nature_valeur_reelle = individu('avantage_en_nature_valeur_reelle', period)
        avantage_en_nature_valeur_forfaitaire = individu('avantage_en_nature_valeur_forfaitaire', period)

        return avantage_en_nature_valeur_reelle + avantage_en_nature_valeur_forfaitaire


class avantage_en_nature_valeur_forfaitaire(Variable):
    value_type = float
    entity = Individu
    label = 'Evaluation fofaitaire des avantages en nature '
    definition_period = MONTH
    set_input = set_input_divide_by_period

    # TODO: coplete this function
    def formula(individu, period, parameters):
        period = period
        avantage_en_nature_valeur_reelle = individu('avantage_en_nature_valeur_reelle', period)

        return avantage_en_nature_valeur_reelle * 0


class depense_cantine_titre_restaurant_employe(Variable):
    value_type = float
    entity = Individu
    label = "Dépense de cantine et de titre restaurant à charge de l'employe"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        period = period

        valeur_unitaire = individu('titre_restaurant_valeur_unitaire', period)
        volume = individu('titre_restaurant_volume', period)
        taux_employeur = individu('titre_restaurant_taux_employeur', period)

        return - valeur_unitaire * volume * (1 - taux_employeur)


class depense_cantine_titre_restaurant_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Dépense de cantine et de titre restaurant à charge de l'employeur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        period = period
        valeur_unitaire = individu('titre_restaurant_valeur_unitaire', period)
        volume = individu('titre_restaurant_volume', period)  # Compute with jours ouvrables ?
        taux_employeur = individu('titre_restaurant_taux_employeur', period)

        return valeur_unitaire * volume * taux_employeur


class nombre_jours_calendaires(Variable):
    value_type = float
    entity = Individu
    label = 'Nombre de jours calendaires travaillés'
    definition_period = MONTH
    default_value = 30
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        contrat_de_travail_debut = individu('contrat_de_travail_debut', period)
        contrat_de_travail_fin = individu('contrat_de_travail_fin', period)

        busday_count = partial(original_busday_count, weekmask = '1' * 7)
        debut_mois = datetime64(period.start.offset('first-of', 'month'))
        fin_mois = datetime64(period.start.offset('last-of', 'month'))
        jours_travailles = max_(
            busday_count(
                max_(contrat_de_travail_debut, debut_mois),
                min_(contrat_de_travail_fin, fin_mois) + timedelta64(1, 'D')
                ),
            0,
            )

        return jours_travailles


class remboursement_transport(Variable):
    value_type = float
    entity = Individu
    label = "Remboursement partiel des frais de transport par l'employeur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):

        remboursement_transport_base = individu('remboursement_transport_base', period)
        # TODO: paramètres en dur dans le code
        return - .5 * remboursement_transport_base


# Fonction publique

class gipa(Variable):
    value_type = float
    entity = Individu
    label = "Indemnité de garantie individuelle du pouvoir d'achat"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # TODO: à coder


class indemnite_residence(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnité de résidence des fonctionnaires'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        salaire_de_base = individu('salaire_de_base', period)
        categorie_salarie = individu('categorie_salarie', period)
        zone_apl = individu.menage('zone_apl', period)
        TypesZoneApl = zone_apl.possible_values
        indemnite_residence = parameters(period).marche_travail.remuneration_dans_fonction_publique.indemnite_residence
        (min_zone_1, min_zone_2, min_zone_3) = (
            indemnite_residence.min * indemnite_residence.taux.zone1,
            indemnite_residence.min * indemnite_residence.taux.zone2,
            indemnite_residence.min * indemnite_residence.taux.zone3
            )
        taux = select(
            [
                (zone_apl == TypesZoneApl.zone_1),
                (zone_apl == TypesZoneApl.zone_2),
                (zone_apl == TypesZoneApl.zone_3),
                ],
            [
                indemnite_residence.taux.zone1,
                indemnite_residence.taux.zone2,
                indemnite_residence.taux.zone3,
                ],
            default = 0
            )
        plancher = select(
            [
                (zone_apl == TypesZoneApl.zone_1),
                (zone_apl == TypesZoneApl.zone_2),
                (zone_apl == TypesZoneApl.zone_3),
                ],
            [
                min_zone_1,
                min_zone_2,
                min_zone_3,
                ],
            default = 0
            )
        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)
            )
        return max_(
            plancher,
            taux * (traitement_indiciaire_brut + salaire_de_base)
            ) * public


class indice_majore(Variable):
    value_type = float
    entity = Individu
    label = 'Indice majoré'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        period = period.start.period('month').offset('first-of')
        categorie_salarie = individu('categorie_salarie', period)
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        traitement_annuel_brut = parameters(period).prestations_sociales.fonc.IM_100
        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )

        return (traitement_indiciaire_brut * 100 * 12 / traitement_annuel_brut) * public


class primes_fonction_publique(Variable):
    value_type = float
    entity = Individu
    label = 'Calcul des primes pour les fonctionnaires'
    reference = 'http://vosdroits.service-public.fr/particuliers/F465.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        # period = period.first_month
        categorie_salarie = individu('categorie_salarie', period)
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)

        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )

        return TAUX_DE_PRIME * traitement_indiciaire_brut * public


class af_nbenf_fonc(Variable):
    value_type = int
    label = "Nombre d'enfants dans la famille au sens des allocations familiales pour les fonctionnaires"
    entity = Famille
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        '''
            Cette variable est une version légèrement modifiée de `af_nbenf`. Elle se base sur le salaire de base, tandis que `af_nbenf` se base sur le salaire net.
            On ne peut pas utiliser la variable `af_nbenf` dans le calcul de `supplement_familial_traitement` (ci-dessous) car `af_nbenf` dépend du `salaire_net`, et `salaire_net` dépends de `supplement_familial_traitement`. Cela créerait une boucle infinie.
            D'où l'introduction de cette variable alternative.
        '''

        salaire_de_base_mensualise = famille.members('salaire_de_base', period.start.period('month', 6).offset(-6), options = [ADD])
        law = parameters(period)
        nbh_travaillees = 169
        smic_mensuel_brut = law.marche_travail.salaire_minimum.smic.smic_b_horaire * nbh_travaillees

        autonomie_financiere = (salaire_de_base_mensualise >= (
            law.prestations_sociales.prestations_familiales.def_pac.revenu_plafond_pac_non_scolaire
            * smic_mensuel_brut
            ))

        age = famille.members('age', period)

        condition_enfant = (
            (age >= law.prestations_sociales.prestations_familiales.prestations_generales.af.af_cm.age1)
            * (age <= law.prestations_sociales.prestations_familiales.prestations_generales.af.af_cm.age2)
            * not_(autonomie_financiere)
            )

        return famille.sum(condition_enfant, role = Famille.ENFANT)


class supplement_familial_traitement(Variable):
    value_type = float
    entity = Individu
    label = 'Supplément familial de traitement'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # Attention : par hypothèse ne peut êre attribué qu'à la tête du ménage
    # TODO: gérer le cas encore problématique du conjoint fonctionnaire

    def formula(individu, period, parameters):
        categorie_salarie = individu('categorie_salarie', period)
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        fonction_publique = parameters(period).marche_travail.remuneration_dans_fonction_publique
        indice_majore_100 = 100 * fonction_publique.indicefp.point_indice_en_nominal

        fonc_nbenf = individu.famille('af_nbenf_fonc', period) * individu.has_role(Famille.DEMANDEUR)

        sft = fonction_publique.sft

        part_fixe = (
            sft.part_fixe.un_enfant * (fonc_nbenf == 1)
            + sft.part_fixe.deux_enfants * (fonc_nbenf >= 2)
            + sft.part_fixe.enfant_supplementaire * max_(0, fonc_nbenf - 2)
            )

        pct_variable = (
            sft.part_proportionnelle.deux_enfants * (fonc_nbenf == 2)
            + sft.part_proportionnelle.trois_enfants * (fonc_nbenf >= 3)
            + sft.part_proportionnelle.enfant_supplementaire * max_(0, fonc_nbenf - 3)
            )

        indice_maj_min = sft.im_plancher
        indice_maj_max = sft.im_plafond

        traitement_brut_mensuel_min = _traitement_brut_mensuel(indice_maj_min, indice_majore_100)
        plancher = part_fixe + traitement_brut_mensuel_min * pct_variable

        traitement_brut_mensuel_max = _traitement_brut_mensuel(indice_maj_max, indice_majore_100)
        plafond = part_fixe + traitement_brut_mensuel_max * pct_variable

        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire)
            )

        sft = public * min_(
            max_(part_fixe + pct_variable * traitement_indiciaire_brut, plancher),
            plafond
            )

        return sft


def _traitement_brut_mensuel(indice_maj, indice_majore_100_annuel):
    traitement_brut = indice_majore_100_annuel * indice_maj / 100 / 12
    return traitement_brut


class remuneration_principale(Variable):
    value_type = float
    entity = Individu
    label = 'Rémunération principale des agents titulaires de la fonction publique'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    unit = 'currency'

    def formula(individu, period, parameters):
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut', period)
        nouvelle_bonification_indiciaire = individu('nouvelle_bonification_indiciaire', period)
        categorie_salarie = individu('categorie_salarie', period)

        public = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_militaire)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_territoriale)
            + (categorie_salarie == TypesCategorieSalarie.public_titulaire_hospitaliere)
            )

        return (
            public * (
                traitement_indiciaire_brut + nouvelle_bonification_indiciaire
                )
            )


class salaire_net_a_payer(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire net à payer (fiche de paie)'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''
        Calcul du salaire net à payer après déduction des sommes
        dues par les salarié avancées par l'employeur
        '''
        salaire_net = individu('salaire_net', period, options = [ADD])
        depense_cantine_titre_restaurant_employe = individu(
            'depense_cantine_titre_restaurant_employe', period)
        indemnites_forfaitaires = individu('indemnites_forfaitaires', period)
        remuneration_apprenti = individu('remuneration_apprenti', period)
        stage_gratification = individu('stage_gratification', period)
        salaire_net_a_payer = (
            salaire_net
            + remuneration_apprenti
            + stage_gratification
            + depense_cantine_titre_restaurant_employe
            + indemnites_forfaitaires
            )
        return salaire_net_a_payer


class salaire_super_brut_hors_allegements(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire super-brut (fiche de paie): rémunération + cotisations sociales employeur'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        salaire_de_base = individu('salaire_de_base', period)
        remuneration_principale = individu('remuneration_principale', period)
        remuneration_apprenti = individu('remuneration_apprenti', period)

        primes_fonction_publique = individu('primes_fonction_publique', period)
        indemnite_residence = individu('indemnite_residence', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        cotisations_employeur = individu('cotisations_employeur', period)
        depense_cantine_titre_restaurant_employeur = individu('depense_cantine_titre_restaurant_employeur', period)
        reintegration_titre_restaurant_employeur = individu('reintegration_titre_restaurant_employeur', period)
        indemnite_fin_contrat = individu('indemnite_fin_contrat', period)
        primes_salaires_non_exonerees = individu('primes_salaires_non_exonerees', period)
        salaire_super_brut_hors_allegements = (
            salaire_de_base
            + remuneration_apprenti
            + indemnite_fin_contrat
            + remuneration_principale
            + primes_fonction_publique
            + indemnite_residence
            + supplement_familial_traitement
            + depense_cantine_titre_restaurant_employeur
            - reintegration_titre_restaurant_employeur
            - cotisations_employeur
            + primes_salaires_non_exonerees
            )

        return salaire_super_brut_hors_allegements


class salaire_super_brut(Variable):
    value_type = float
    entity = Individu
    label = 'Coût du travail à court terme. Inclut les exonérations et allègements de charges'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        period = period
        salaire_super_brut_hors_allegements = individu('salaire_super_brut_hors_allegements', period)
        exonerations_et_allegements = individu('exonerations_et_allegements', period)
        return salaire_super_brut_hors_allegements - exonerations_et_allegements

    def formula_2019_01_01(individu, period, parameters):
        period = period
        salaire_super_brut_hors_allegements = individu('salaire_super_brut_hors_allegements', period)
        exonerations_et_allegements = individu('exonerations_et_allegements', period)
        prime_exceptionnelle_pouvoir_achat_exoneree = individu('prime_exceptionnelle_pouvoir_achat_exoneree', period, options = [DIVIDE])
        return salaire_super_brut_hors_allegements - exonerations_et_allegements + prime_exceptionnelle_pouvoir_achat_exoneree

    def formula_2022_07_01(individu, period, parameters):
        '''
        Apparition de la PPV le 1er aout 2022:
        Au niveau du salaire super brut, la PPV se comporte comme la PEPA.
        Pour tout niveau de revenu, la part de prime sous plafond est exonérée
        de toutes les cotisations sociales d'origine légale ou conventionnelle
        à la charge du salarié et de l'employeur (mais pas de contribution
        "forfait social") et la part de la prime non exonérée
        (sur plafond) est soumise aux cotisations.
        '''
        period = period
        salaire_super_brut_hors_allegements = individu('salaire_super_brut_hors_allegements', period)
        exonerations_et_allegements = individu('exonerations_et_allegements', period)
        prime_partage_valeur_exoneree = individu('prime_partage_valeur_exoneree', period, options=[DIVIDE])
        prime_partage_valeur_exoneree_exceptionnelle = individu('prime_partage_valeur_exoneree_exceptionnelle', period, options=[DIVIDE])
        return (
            salaire_super_brut_hors_allegements
            - exonerations_et_allegements
            + prime_partage_valeur_exoneree
            + prime_partage_valeur_exoneree_exceptionnelle
            )


class exonerations_et_allegements(Variable):
    value_type = float
    entity = Individu
    label = 'Exonérations et allègements'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        exoneration_cotisations_employeur_apprenti = individu(
            'exoneration_cotisations_employeur_apprenti', period, options = [ADD])
        exoneration_cotisations_employeur_geographiques = individu(
            'exoneration_cotisations_employeur_geographiques', period)
        exoneration_cotisations_employeur_tode = individu(
            'exoneration_cotisations_employeur_tode', period)
        exoneration_cotisations_employeur_jei = individu(
            'exoneration_cotisations_employeur_jei', period, options = [ADD])
        exoneration_cotisations_employeur_stagiaire = individu(
            'exoneration_cotisations_employeur_stagiaire', period, options = [ADD])

        allegement_fillon = individu('allegement_fillon', period, options = [ADD])
        allegement_cotisation_maladie = individu('allegement_cotisation_maladie', period, options = [ADD])
        allegement_cotisation_allocations_familiales = individu('allegement_cotisation_allocations_familiales', period, options = [ADD])

        return (
            allegement_fillon
            + allegement_cotisation_maladie
            + allegement_cotisation_allocations_familiales
            + exoneration_cotisations_employeur_tode
            + exoneration_cotisations_employeur_geographiques
            + exoneration_cotisations_employeur_jei
            + exoneration_cotisations_employeur_apprenti
            + exoneration_cotisations_employeur_stagiaire
            )


class cout_du_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Coût du travail à long terme. Inclut les charges, aides et crédits différés'
    set_input = set_input_divide_by_period
    definition_period = MONTH
    calculate_output = calculate_output_add

    def formula(individu, period, parameters):
        salaire_super_brut = individu('salaire_super_brut', period)
        cout_differe = individu('cout_differe', period)
        return salaire_super_brut - cout_differe


class cout_differe(Variable):
    value_type = float
    entity = Individu
    label = 'Charges, aides et crédits différés ou particuliers'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        credit_impot_competitivite_emploi = individu('credit_impot_competitivite_emploi', period, options = [ADD])
        aide_premier_salarie = individu('aide_premier_salarie', period, options = [ADD])
        aide_embauche_pme = individu('aide_embauche_pme', period, options = [ADD])
        tehr = individu('tehr', period, options = [DIVIDE])

        return credit_impot_competitivite_emploi + aide_premier_salarie + aide_embauche_pme + tehr


class TypesConges(Enum):
    non_renseigne = 'Non renseigné'
    conge_parental = 'Congé parental'
    conge_maternite_paternite = 'Congé maternité ou paternité'
    conge_presence_parentale = 'Congé de présence parentale'
    conge_conventionnel = 'Congé conventionnel'
    conge_sans_solde = 'Congé sans solde'
    disponibilite = 'Mise en disponibilité (fonction publique)'
    conge_sabbatique = 'Congé sabbatique'


class type_conges(Variable):
    value_type = Enum
    possible_values = TypesConges
    default_value = TypesConges.non_renseigne
    entity = Individu
    label = 'Type de congés en cours'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesTaches(Enum):
    # https://www.msa.fr/lfp/employeur/exonerations-travailleurs-occasionnels (bénéficiaires)
    non_renseigne = 'Non renseigné'
    production_animale_vegetale = 'Tâches dans les activités liées au cycle de la production animale et végétale'
    prolongement_production = "Tâches dans les activités constituant le prolongement direct de l'acte de production"
    travaux_forestiers = 'Tâches dans les activités liées aux travaux forestiers'


class taches_salarie_type(Variable):
    value_type = Enum
    possible_values = TypesTaches
    default_value = TypesTaches.non_renseigne
    entity = Individu
    label = 'Type des tâches affectées au salarié'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class travailleur_occasionnel_agricole(Variable):
    value_type = bool
    entity = Individu
    label = 'Le salarié est travailleur occasionnel agricole'
    definition_period = MONTH
    reference = 'https://www.msa.fr/lfp/employeur/exonerations-travailleurs-occasionnels'
    set_input = set_input_dispatch_by_period
    documentation = '''
    Sont considérés comme "travailleurs occasionnels agricoles", les salariés qui remplissent deux conditions
    se rapportant à la nature de leur contrat de travail et à la nature des tâches affectées.

    Cas non modélisé (2022): CDI conclu avec un demandeur d'emploi (inscrit à Pôle emploi depuis au moins 4 mois ou 1 mois
    si cette inscription fait suite à un licenciement) par un groupement d'employeurs composés exclusivement de membres
    exerçant les activités éligibles (cycle de la production animale et végétale, travaux forestiers,
    activités constituant le prolongement direct de l'acte de production).
    '''

    def formula(individu, period):
        secteur_agricole = individu('secteur_activite_employeur', period) == TypesSecteurActivite.agricole
        cdd = individu('contrat_de_travail_type', period) == TypesContrat.cdd
        contrat_duree_determinee_type = individu('contrat_duree_determinee_type', period)

        cdd_occasionnel_agricole = (
            (contrat_duree_determinee_type == TypesContratTravailDureeDeterminee.contrat_saisonnier)
            + (contrat_duree_determinee_type == TypesContratTravailDureeDeterminee.contrat_usage)
            + (contrat_duree_determinee_type == TypesContratTravailDureeDeterminee.contrat_vendanges)
            + (contrat_duree_determinee_type == TypesContratTravailDureeDeterminee.contrat_insertion)
            + (contrat_duree_determinee_type == TypesContratTravailDureeDeterminee.contrat_initiative_emploi)
            )

        taches_salarie_type = individu('taches_salarie_type', period)
        taches_eligibles = (
            (taches_salarie_type == TypesTaches.production_animale_vegetale)
            + (taches_salarie_type == TypesTaches.prolongement_production)
            + (taches_salarie_type == TypesTaches.travaux_forestiers)
            )
        return secteur_agricole * cdd * cdd_occasionnel_agricole * taches_eligibles



from numpy import logical_not as not_
from openfisca_france.model.base import (
    Variable,
    Individu,
    Famille,
    MONTH,
    set_input_divide_by_period,
    )


class acs_montant_i(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'ACS attribué à une personne en cas d'éligibilité de la famille"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2005_01_01(individu, period, parameters):
        age = individu('age', period)
        bareme = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.acs.bareme

        return (age >= 0) * bareme.calc(age)


class acs_montant(Variable):
    value_type = float
    entity = Famille
    label = "Montant de l'ACS attribué à une famille en cas d'éligibilité"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2009_08_01(famille, period, parameters):
        acs_montant_i = famille.members('acs_montant_i', period)
        return famille.sum(acs_montant_i)


class acs_plafond(Variable):
    value_type = float
    entity = Famille
    label = "Plafond annuel de ressources pour l'éligibilité à l'ACS"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.acs
        cmu_c_plafond = famille('cmu_c_plafond', period)

        return cmu_c_plafond * (1 + P.majoration_plafond_acs)


class acs(Variable):
    value_type = float
    label = "Montant (annuel) de l'ACS"
    entity = Famille
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2021-05-01'

    def formula_2019_11_01(famille, period):
        return

    def formula(famille, period):
        cmu_c = famille('cmu_c', period)
        css_cmu_base_ressources = famille('css_cmu_base_ressources', period)
        acs_plafond = famille('acs_plafond', period)
        acs_montant = famille('acs_montant', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)
        css_cmu_acs_eligibilite = famille('css_cmu_acs_eligibilite', period)

        return (
            css_cmu_acs_eligibilite
            * not_(residence_mayotte)
            * not_(cmu_c)
            * (css_cmu_base_ressources <= acs_plafond)
            * acs_montant
            )



from typing import Callable, Tuple

import pytest

from ..cache import tax_benefit_system
from openfisca_core.simulations import Simulation
from openfisca_france.reforms.plf2015 import plf2015


@pytest.fixture
def simulations(new_scenario) -> Callable[..., Tuple[Simulation]]:
    def _simulations(year: int) -> Tuple[Simulation]:
        scenario = new_scenario(
            reform = plf2015(tax_benefit_system),
            count = 2,
            _max = 18000,
            _min = 0,
            name = 'salaire_imposable',
            year = year,
            people = 1,
            )

        return scenario.new_simulation(use_baseline = True), scenario.new_simulation()

    return _simulations


def test_plf2015_impots_directs(simulations, year = 2013, error = 1):
    actual_simulation, reform_simulation = simulations(year)
    actual_impots = actual_simulation.calculate('impots_directs', year)
    reform_impots = reform_simulation.calculate('impots_directs', year)
    assert max(abs([0, -869] - actual_impots)) < error
    assert max(abs([0, -911.4 + (1135 - 911.4)] - reform_impots)) < error


def test_plf2015_ir_plaf_qf(simulations, year = 2013, error = 1):
    actual_simulation, reform_simulation = simulations(year)
    actual_ir_plaf_qf = actual_simulation.calculate('ir_plaf_qf', year)
    reform_ir_plaf_qf = reform_simulation.calculate('ir_plaf_qf', year)
    assert max(abs([0, 918] - actual_ir_plaf_qf)) < error
    assert max(abs([0, 911.4] - reform_ir_plaf_qf)) < error




import datetime


from openfisca_core import periods

from openfisca_france.scenarios import init_single_entity

from openfisca_core.tools import assert_near
from openfisca_france.reforms.trannoy_wasmer import trannoy_wasmer
from ..cache import tax_benefit_system


def test_charge_loyer():
    year = 2013
    reform = trannoy_wasmer(tax_benefit_system)
    scenario = init_single_entity(reform.new_scenario(),
        axes = [[
            dict(
                count = 10,
                max = 30000,
                min = 0,
                name = 'salaire_de_base',
                ),
            ]],
        period = periods.period(year),
        parent1 = dict(date_naissance = datetime.date(year - 40, 1, 1)),
        parent2 = dict(date_naissance = datetime.date(year - 40, 1, 1)),
        enfants = [
            dict(date_naissance = datetime.date(year - 9, 1, 1)),
            dict(date_naissance = datetime.date(year - 9, 1, 1)),
            ],
        menage = dict(
            loyer = 1000,
            ),
        )
    reform_simulation = scenario.new_simulation()
    absolute_error_margin = 0.01

    reform_charge_loyer = reform_simulation.calculate('charge_loyer', period = year)
    assert_near(reform_charge_loyer, [1000] * 10, absolute_error_margin = absolute_error_margin)


if __name__ == '__main__':
    import logging
    import sys
    logging.basicConfig(level = logging.ERROR, stream = sys.stdout)
    test_charge_loyer()



#! /usr/bin/env python
import time
from openfisca_core.simulation_builder import SimulationBuilder
from openfisca_france import FranceTaxBenefitSystem
from openfisca_france.situation_examples import couple

tax_benefit_system = FranceTaxBenefitSystem()

NB_RUN = 10

situation = {
    'individus': {
        'demandeur': {
            'salaire_imposable': {
                '2017-12': 62191,
                '2018-01': 62191,
                '2018-02': 62191,
                '2018-03': 62191,
                '2018-04': 62191,
                '2018-05': 62191,
                '2018-06': 62191,
                '2018-07': 62191,
                '2018-08': 62191,
                '2018-09': 62191,
                '2018-10': 62191,
                '2018-11': 62191,
                '2018-12': 62191,
                '2016-01': 62191,
                '2016-02': 62191,
                '2016-03': 62191,
                '2016-04': 62191,
                '2016-05': 62191,
                '2016-06': 62191,
                '2016-07': 62191,
                '2016-08': 62191,
                '2016-09': 62191,
                '2016-10': 62191,
                '2016-11': 62191,
                '2016-12': 62191
                }
            }
        },
    'familles': {
        '_': {
            'parents': [
                'demandeur'
                ]
            }
        },
    'foyers_fiscaux': {
        '_': {
            'declarants': [
                'demandeur'
                ],
            'personnes_a_charge': []
            }
        },
    'menages': {
        '_': {
            'personne_de_reference': [
                'demandeur'
                ]
            }
        }
    }


def timeit(method):
    def timed(*args, **kwargs):
        start_time = time.time()
        result = method(*args, **kwargs)
        delta = time.time() - start_time
        return result, delta

    return timed


def test_revenu_disponible():
    def new_simulation():
        return SimulationBuilder().build_from_dict(tax_benefit_system, couple)

    simulations = [new_simulation() for i in range(NB_RUN)]

    @timeit
    def run_test():
        for simulation in simulations:
            simulation.calculate('revenu_disponible', 2018)

    result, delta = run_test()
    print('{:2.6f} s'.format(delta / NB_RUN))  # noqa T201


def test_spiral():
    def new_simulation():
        return SimulationBuilder().build_from_dict(tax_benefit_system, situation)

    simulations = [new_simulation() for i in range(NB_RUN)]

    @timeit
    def run_test():
        for simulation in simulations:
            simulation.calculate('ass', '2018-12')
            simulation.calculate('logement_social_eligible', '2018-12')

    result, delta = run_test()
    print('{:2.6f} s'.format(delta / NB_RUN))  # noqa T201


print('Premier test revenu disponible')  # noqa T201
test_revenu_disponible()
print('Second test revenu disponible')  # noqa T201
test_revenu_disponible()
print('3e test revenu disponible')  # noqa T201
test_revenu_disponible()

print('Premier test ciblé spirale')  # noqa T201
test_spiral()
print('Second test ciblé spirale')  # noqa T201
test_spiral()
print('3e test ciblé spirale')  # noqa T201
test_spiral()



from openfisca_france.model.base import *


class covid_aide_exceptionnelle_tpe_eligible(Variable):
    entity = Individu
    value_type = bool
    label = "Éligibilité à l'aide exceptionnelle pour les TPE pendant la crise sanitaire dûe au COVID-19"
    reference = [
        'Décret n°2020-371 du 30 mars 2020'
        'https://www.legifrance.gouv.fr/eli/decret/2020/3/30/ECOI2007755D/jo/texte',
        'Ordonnance n° 2020-705 du 10 juin 2020'
        'https://www.legifrance.gouv.fr/eli/ordonnance/2020/6/10/ECOI2012371R/jo/texte'
        ]
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2020_03(individu, period):
        chiffre_d_affaire = individu('rpns_auto_entrepreneur_chiffre_affaires', period)
        period_1 = period.offset(-1, 'year')
        chiffre_d_affaire_annee_mois_un_an_avant = individu('rpns_auto_entrepreneur_chiffre_affaires', period_1)
        chiffre_d_affaire_annee_moyenne_annee_precedente = individu('rpns_auto_entrepreneur_chiffre_affaires', period.last_year, options = [ADD]) / 12
        chiffre_d_affaire_annee_n_1 = max_(chiffre_d_affaire_annee_mois_un_an_avant, chiffre_d_affaire_annee_moyenne_annee_precedente)
        return individu('travailleur_non_salarie', period) * (((chiffre_d_affaire - chiffre_d_affaire_annee_n_1) / chiffre_d_affaire_annee_n_1) < -0.5)


class covid_aide_exceptionnelle_tpe_montant(Variable):
    entity = Individu
    value_type = float
    label = "Montant de l'aide exceptionnelle pour les TPE pendant la crise sanitaire dûe au COVID-19"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2020-12-31'

    def formula_2020_03(individu, period, parameters):
        eligibilite_fse = individu('covid_aide_exceptionnelle_tpe_eligible', period)
        plafond_fse = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.covid19.aide_exceptionnelle_tpe.plafond
        chiffre_d_affaire = individu('rpns_auto_entrepreneur_chiffre_affaires', period)
        period_1 = period.offset(-1, 'year')
        chiffre_d_affaire_annee_mois_un_an_avant = individu('rpns_auto_entrepreneur_chiffre_affaires', period_1)
        chiffre_d_affaire_annee_moyenne_annee_precedente = individu('rpns_auto_entrepreneur_chiffre_affaires', period.last_year, options = [ADD]) / 12
        chiffre_d_affaire_annee_n_1 = max_(chiffre_d_affaire_annee_mois_un_an_avant, chiffre_d_affaire_annee_moyenne_annee_precedente)
        difference_chiffre_d_affaire = chiffre_d_affaire - chiffre_d_affaire_annee_n_1
        return eligibilite_fse * (difference_chiffre_d_affaire < 0) * min_(plafond_fse, -difference_chiffre_d_affaire)


class covid_activite_partielle_eligible(Variable):
    entity = Individu
    value_type = bool
    label = 'Eligibilité au dispositif du chômage partiel'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        return individu('salaire_de_base', period) != 0


class covid_activite_partielle_montant(Variable):
    entity = Individu
    value_type = float
    label = "Montant de l'indemnité salariale de chômage partiel"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        "Ordonnance n° 2020-346 du 27 mars 2020 portant mesures d'urgence en matière d'activité partielle",
        'https://www.legifrance.gouv.fr/eli/ordonnance/2020/3/27/MTRX2008381R/jo/texte',
        ]

    def formula_2020_03(individu, period, parameters):
        '''
        Il s'agit de l'indemnité et pas de l'allocation (somme que doit verser l'entreprise au salarié et pas la somme que l'Etat verse à l'entreprise.
        '''
        eligibilite_activite_partielle = individu('covid_activite_partielle_eligible', period)
        heures = individu('heures_remunerees_volume', period)
        salaire_horaire = individu('salaire_de_base', period) / heures
        indemnite_ap = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.covid19.indemnite_ap
        indemnite_horaire = max(
            indemnite_ap.taux * salaire_horaire,
            indemnite_ap.plancher
            )
        return eligibilite_activite_partielle * indemnite_horaire * heures



from openfisca_france.model.base import *
from numpy import datetime64


class visale_eligibilite(Variable):
    value_type = bool
    entity = Menage
    label = "Indique l'éligibilité à une caution Visale"
    definition_period = MONTH
    reference = 'https://www.visale.fr/vos-questions/faq-locataires/locataire-de-30-ans-ou-moins-suis-je-eligible/'

    # L'extension à toutes les personnes de moins de 30 ans sauf les étudiants boursiers encore rattachés au foyer fiscal de leurs parents a été faite en septembre 2016 : https://www.lemonde.fr/immobilier/article/2016/08/04/location-la-caution-visale-etendue-aux-moins-de-30-ans-au-plus-tard-le-30-septembre_4978567_1306281.html
    # L'extension à toutes les personnes de moins de 30 ans y compris les étudiants boursiers encore rattachés au foyer fiscal de leurs parents a été faite le 19 juin 2018.
    def formula_2018_06_19(menage, period, parameters):
        '''
        Le cas où un ménage est constitué d'une personne éligible et l'autre non éligible n'est pas spécifié dans la documentation Visale, on va donc tester l'égibilité uniquement sur la personne de référence.
        La documentation Visale indique : « Si vous êtes 2 à rechercher un logement et un garant, votre demande Visale doit être réalisée conjointement. Pour les logements en colocation, au-delà de 2 colocataires, un bail et un visa individuels doivent être faits par colocataire. »
        Cette modélisation est impossible à réaliser telle quelle dans OpenFisca, car cela correspondrait à une variable de Ménage pour 1 à 2 personnes, et une variable d'Individu à partir de 3 personnes en colocation, mais pour lesquelles le montant du loyer serait différent (ou en tous cas, serait la quote-part du loyer total du logement loué).
        Par conséquent, le calcul de cette variable fait l'hypothèse d'une déclaration des Ménages avec un Ménage par personne inscrite sur le bail pour 3 personnes ou plus, et avec un seul Ménage pour une colocation (ou un bail solidaire) de 2 personnes.
        '''
        age = menage.personne_de_reference('age', period)
        majeur = menage.personne_de_reference('majeur', period)

        eligibilite_age = majeur * (age <= parameters(period).prestations_sociales.aides_logement.action_logement.visale.eligibilite.age_max)

        etudiant = menage.personne_de_reference('etudiant', period)
        ressortissant_eee = menage.personne_de_reference('ressortissant_eee', period)
        nationalite = menage.personne_de_reference('nationalite', period)
        ressortissant_pays_eligible = sum([nationalite == str.encode(etat) for etat in parameters(period).prestations_sociales.aides_logement.action_logement.visale.eligibilite.residence_hors_eee])  # TOOPTIMIZE: string encoding into bytes array should be done at load time

        eligibilite_nationalite = ressortissant_eee + ressortissant_pays_eligible + etudiant  # Sont éligibles les « étudiant‧e‧s hors Union Européenne justifiant d’un visa long séjour valant titre de séjour mention étudiant ou passeport talent en cours de validité, ou d’un titre de séjour mention étudiant en cours de validité ». Vu qu'il s'agit d'une aide au logement, on suppose que le visa long séjour (4 mois à 1 an) est acquis, et on ignore donc les cas où un étudiant hors UE vient pour une durée de moins de 4 mois sans visa (ex : étudiante néo-zélandaise en visa touristique de 90 jours, l'éligibilité à Visale sera indiquée à tort comme positive).

        loyer = menage('loyer', period)
        charges_locatives = menage('charges_locatives', period)
        montant_max = menage('visale_montant_max', period)

        eligibilite_loyer = (loyer + charges_locatives) <= montant_max
        eligibilite_date_entree_logement = menage('date_entree_logement', period) > datetime64(period.start)

        return eligibilite_age * eligibilite_nationalite * eligibilite_loyer * eligibilite_date_entree_logement


class visale_montant_max(Variable):
    value_type = float
    entity = Menage
    label = 'Montant maximum du loyer éligible à une caution Visale'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.visale.fr/vos-questions/faq-locataires/locataire-de-30-ans-ou-moins-suis-je-eligible/#13'

    def formula_2016_01_01(menage, period, parameters):
        '''
        Attention, un montant non nul pour cette variable ne signifie pas nécessairement que l'entité est éligible à Visale : d'autres conditions peuvent ne pas être remplies. Pour déterminer l'éligibilité à la caution Visale au loyer actuellement renseigné pour le ménage, il faut utiliser la variable `visale_eligibilite`.
        La documentation Visale indique : « Si vous êtes 2 à rechercher un logement et un garant, votre demande Visale doit être réalisée conjointement. Pour les logements en colocation, au-delà de 2 colocataires, un bail et un visa individuels doivent être faits par colocataire. »
        Cette modélisation est impossible à réaliser telle quelle dans OpenFisca, car cela correspondrait à une variable de Ménage pour 1 à 2 personnes, et une variable d'Individu à partir de 3 personnes en colocation, mais pour lesquelles le montant du loyer serait différent (ou en tous cas, serait la quote-part du loyer total du logement loué).
        Par conséquent, le calcul de cette variable fait l'hypothèse d'une déclaration des Ménages avec un Ménage par personne inscrite sur le bail pour 3 personnes ou plus, et avec un seul Ménage pour une colocation (ou un bail solidaire) de 2 personnes.
        '''
        residence_ile_de_france = menage('residence_ile_de_france', period)

        etudiant = menage.personne_de_reference('etudiant', period)  # le cas où un ménage est constitué d'une personne étudiante et d'une personne non étudiante n'est pas spécifié dans la documentation Visale
        minimum_etudiant = where(
            residence_ile_de_france,
            parameters(period).prestations_sociales.aides_logement.action_logement.visale.plafond_loyer.etudiant.ile_de_france,
            parameters(period).prestations_sociales.aides_logement.action_logement.visale.plafond_loyer.etudiant.hors_ile_de_france,
            )

        plafond_loyer = where(
            residence_ile_de_france,
            parameters(period).prestations_sociales.aides_logement.action_logement.visale.plafond_loyer.cas_general.ile_de_france,
            parameters(period).prestations_sociales.aides_logement.action_logement.visale.plafond_loyer.cas_general.hors_ile_de_france,
            )

        moitie_des_ressources = menage('visale_base_ressources', period) / 2

        return max_(etudiant * minimum_etudiant, min_(moitie_des_ressources, plafond_loyer))


class visale_base_ressources(Variable):
    value_type = float
    entity = Menage
    label = "Revenus pris en compte pour l'éligibilité à la caution Visale"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.visale.fr/wp-content/uploads/2020/04/Visale-Ressources-Locataire_2020.pdf#page_7'

    def formula_2016_01_01(menage, period, parameters):
        '''
        La documentation Visale indique : « Si vous êtes 2 à rechercher un logement et un garant, votre demande Visale doit être réalisée conjointement. Pour les logements en colocation, au-delà de 2 colocataires, un bail et un visa individuels doivent être faits par colocataire. »
        Cette modélisation est impossible à réaliser telle quelle dans OpenFisca, car cela correspondrait à une variable de Ménage pour 1 à 2 personnes, et une variable d'Individu à partir de 3 personnes en colocation, mais pour lesquelles le montant du loyer serait différent (ou en tous cas, serait la quote-part du loyer total du logement loué).
        Par conséquent, le calcul de cette variable fait l'hypothèse d'une déclaration des Ménages avec un Ménage par personne inscrite sur le bail pour 3 personnes ou plus, et avec un seul Ménage pour une colocation (ou un bail solidaire) de 2 personnes.
        '''
        revenus_individus = menage.sum(menage.members('visale_base_ressources_individuelle', period))

        ressources_foyer_fiscal = [
            'rente_viagere_titre_onereux',
            ]

        ressources_famille = [
            'aeeh',
            'af',
            'asf',
            'aspa',
            'cf',
            'ppa',
            'rsa',
            ]

        ressources_famille_abattues = [  # ressources de la famille à prendre en compte à 67% sur la base du loyer du logement pour lequel Visale est demandé
            'alf',
            'als',
            'apl',
            ]

        revenus_foyers_fiscaux = sum(
            menage.sum(menage.members.foyer_fiscal(ressource, period.last_month), role = FoyerFiscal.DECLARANT_PRINCIPAL)
            for ressource in ressources_foyer_fiscal
            )

        revenus_familles = sum(
            menage.sum(menage.members.famille(ressource, period.last_month), role = Famille.DEMANDEUR)
            for ressource in ressources_famille
            )

        revenus_familles_abattus = sum(
            menage.sum(menage.members.famille(ressource, period), role = Famille.DEMANDEUR)
            for ressource in ressources_famille_abattues
            ) * parameters(period).prestations_sociales.aides_logement.action_logement.visale.quote_part_aides_logement

        return revenus_individus + revenus_foyers_fiscaux + revenus_familles + revenus_familles_abattus


class visale_base_ressources_individuelle(Variable):
    value_type = float
    entity = Individu
    label = "Revenus des individus pris en compte pour l'éligibilité à la caution Visale"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.visale.fr/wp-content/uploads/2020/04/Visale-Ressources-Locataire_2020.pdf#page_7'

    def formula_2016_01_01(individu, period, parameters):
        ressources_individu_mensuelles = [
            'salaire_net',
            'primes_salaires_non_exonerees',  # Sont prises en compte toutes les « primes récurrentes perçues dans le cadre d’une activité intégrée dans le calcul du salaire ou traitement ». Note : ce revenu est brut, il devrait être net.
            'prime_forfaitaire_mensuelle_reprise_activite',
            'revenus_stage_formation_pro',
            'remuneration_apprenti',
            'garantie_jeunes',
            'bourse_enseignement_sup',
            'bourse_recherche',
            'rpns_auto_entrepreneur_benefice',
            'chomage_net',
            'indemnites_chomage_partiel',  # à inclure dans chomage_net ?
            'ass',
            'retraite_nette',
            'pensions_alimentaires_percues',
            'pensions_invalidite',
            'pch',
            'asi',
            'apa_domicile',
            'indemnites_journalieres',
            'rente_accident_travail',
            # Ressources non prises en compte à ce jour car non modélisées dans OpenFisca :
            # mobili-jeune
            # prime transitoire de solidarité (PTS)
            # allocation journalière de présence parentale (AJPP)
            ]

        revenus_individu = sum(individu(ressource, period.last_month) for ressource in ressources_individu_mensuelles)  # les justificatifs des ressources sont demandés « sur le mois précédant la demande de visa »

        revenus_non_salarie_nets = individu('revenus_non_salarie_nets', period.last_year.first_month, options = [DIVIDE])

        return revenus_individu + revenus_non_salarie_nets



from openfisca_france.model.base import Individu, Menage, TypesActivite, TypesStatutOccupationLogement, Variable, MONTH, set_input_dispatch_by_period
from numpy import datetime64


class locapass_eligibilite(Variable):
    entity = Individu
    value_type = bool
    label = "Indicatrice d'éligibilité au dispositif Loca-Pass"
    definition_period = MONTH
    reference = 'https://www.actionlogement.fr/l-avance-loca-pass'

    def formula(individu, period, parameters):
        params = parameters(period).prestations_sociales.aides_logement.action_logement.locapass

        eligibilite_individu = individu('locapass_eligibilite_individu', period)
        eligibilite_logement = individu.menage('locapass_eligibilite_logement', period)
        eligibilite_date_entree_logement = individu.menage('date_entree_logement', period) >= datetime64(period.offset(-params.delai_max, 'month').start)
        return eligibilite_individu * eligibilite_logement * eligibilite_date_entree_logement


class locapass_eligibilite_logement(Variable):
    entity = Menage
    value_type = bool
    label = "Satisfaction des conditions d'éligibilité relatives au logement pour le dispositif Loca-Pass"
    definition_period = MONTH

    def formula(menage, period):
        statut_occupation = menage('statut_occupation_logement', period)
        return (
            + (statut_occupation == TypesStatutOccupationLogement.locataire_hlm)
            + (statut_occupation == TypesStatutOccupationLogement.locataire_vide)
            + (statut_occupation == TypesStatutOccupationLogement.locataire_meuble)
            + (statut_occupation == TypesStatutOccupationLogement.locataire_foyer)
            )


class locapass_eligibilite_individu(Variable):
    entity = Individu
    value_type = bool
    label = "Satisfaction des conditions d'éligibilité relatives au demandeur pour le dispositif Loca-Pass"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        eligibilite_salarie = individu('locapass_eligibilite_salarie', period)
        locapass_eligibilite_jeunes = individu('locapass_eligibilite_jeunes', period)
        locapass_eligibilite_etudiant = individu('locapass_eligibilite_etudiant', period)
        return eligibilite_salarie + locapass_eligibilite_jeunes + locapass_eligibilite_etudiant


class locapass_eligibilite_salarie(Variable):
    entity = Individu
    value_type = bool
    label = "Indicatrice d'éligibilité au dispositif Loca-Pass pour les salariés"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        return individu('activite', period) == TypesActivite.actif


class locapass_eligibilite_jeunes(Variable):
    entity = Individu
    value_type = bool
    label = "Indicatrice d'éligibilité au dispositif Loca-Pass pour les jeunes"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        age = individu('age', period)
        age_max = parameters(period).prestations_sociales.aides_logement.action_logement.locapass.age_max
        eligibilite_age = age <= age_max

        activite = individu('activite', period)
        alternant = individu('alternant', period)
        eligibilite_activite = (activite == TypesActivite.chomeur) + alternant * (activite == TypesActivite.etudiant)

        return eligibilite_age * eligibilite_activite


class locapass_eligibilite_etudiant_contrat(Variable):
    '''
    Approximation des conditions suivantes :
    - d’un contrat à durée déterminée (CDD) de trois mois minimum en cours au moment de la demande d'aide,
    - d’un ou plusieurs CDD pour une durée cumulée de trois mois minimum au cours des six mois précédant la demande d'aide,
    - ou d’une convention de stage d'au moins trois mois en cours au moment de la demande,
    '''
    entity = Individu
    value_type = bool
    label = 'Satisfaction de la condition de loca-pass relative au contrat de travail pour les étudiants'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        nb_mois_travailles = sum([(individu('salaire_net', period.offset(-i)) > 0) for i in range(6)])
        return nb_mois_travailles >= 3


class locapass_eligibilite_etudiant(Variable):
    entity = Individu
    value_type = bool
    label = "Indicatrice d'éligibilité au dispositif Loca-Pass pour les étudiants"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        etudiant = individu('etudiant', period)
        boursier = individu('boursier', period)
        eligibilite_contrat = individu('locapass_eligibilite_etudiant_contrat', period)

        return etudiant * (boursier + eligibilite_contrat)



from openfisca_france.model.base import *  # noqa analysis:ignore

from openfisca_core.reforms import Reform
from openfisca_core.taxscales import MarginalRateTaxScale


TAUX_DE_PRIME = .10


class salaire_imposable_pour_inversion(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire imposable utilisé pour remonter au salaire brut'
    definition_period = MONTH


class salaire_de_base(Variable):
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''Calcule le salaire brut à partir du salaire imposable par inversion du barème
        de cotisations sociales correspondant à la catégorie à laquelle appartient le salarié.
        '''
        # Get value for year and divide below.
        salaire_imposable_pour_inversion = individu('salaire_imposable_pour_inversion',
            period.start.offset('first-of', 'year').period('year'))

        # Calcule le salaire brut à partir du salaire imposable.
        # Sauf pour les fonctionnaires où il renvoie le traitement indiciaire brut
        # Note : le supplément familial de traitement est imposable.

        hsup = individu('hsup', period)
        categorie_salarie = individu('categorie_salarie', period)
        P = parameters(period)

        salarie = P.cotsoc.cotisations_salarie
        plafond_securite_sociale_annuel = P.prelevements_sociaux.pss.plafond_securite_sociale_annuel
        taux_csg = parameters(period).prelevements_sociaux.contributions_sociales.csg.activite.deductible.taux * (1 - .0175)
        csg = MarginalRateTaxScale(name = 'csg')
        csg.add_bracket(0, taux_csg)
        prive_non_cadre = salarie['prive_non_cadre'].combine_tax_scales().scale_tax_scales(
            plafond_securite_sociale_annuel)
        prive_cadre = salarie['prive_cadre'].combine_tax_scales().scale_tax_scales(plafond_securite_sociale_annuel)
        # On ajoute la CSG deductible
        prive_non_cadre.add_tax_scale(csg)
        prive_cadre.add_tax_scale(csg)

        salaire_de_base = (
            (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
            * prive_non_cadre.inverse().calc(salaire_imposable_pour_inversion)
            + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            * prive_cadre.inverse().calc(salaire_imposable_pour_inversion)
            )

        return salaire_de_base + hsup


class traitement_indiciaire_brut(Variable):
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''Calcule le tratement indiciaire brut à partir du salaire imposable.
        '''
        # Get value for year and divide below.
        salaire_imposable_pour_inversion = individu('salaire_imposable_pour_inversion',
            period.start.offset('first-of', 'year').period('year'))

        # Calcule le salaire brut à partir du salaire imposable par inversion numérique.
#            if salaire_imposable_pour_inversion == 0 or (salaire_imposable_pour_inversion == 0).all():
#                # Quick path to avoid fsolve when using default value of input variables.
#                return salaire_imposable_pour_inversion

        # Calcule le salaire brut à partir du salaire imposable.
        # Sauf pour les fonctionnaires où il renvoie le traitement indiciaire brut
        # Note : le supplément familial de traitement est imposable.
        categorie_salarie = individu('categorie_salarie', period)
        P = parameters(period)
        taux_csg = parameters(period).prelevements_sociaux.contributions_sociales.csg.activite.deductible.taux * (1 - .0175)
        csg = MarginalRateTaxScale(name = 'csg')
        csg.add_bracket(0, taux_csg)

        salarie = P.cotsoc.cotisations_salarie
#            cat = None
#            if (categorie_salarie == 2).all():
#                cat = 'public_titulaire_etat'
#            if cat is not None:
#                for name, bareme in salarie[cat].items():
#                    print name, bareme

        # public etat
        # TODO: modifier la contribution exceptionelle de solidarité
        # en fixant son seuil de non imposition dans le barème (à corriger dans param.xml
        # et en tenant compte des éléments de l'assiette
        # salarie['fonc']['etat']['excep_solidarite'] = salarie['fonc']['commun']['solidarite']

        public_titulaire_etat = salarie['public_titulaire_etat']  # .copy()
        public_titulaire_etat['rafp'].multiply_rates(TAUX_DE_PRIME, inplace = True)
        public_titulaire_etat = salarie['public_titulaire_etat'].combine_tax_scales()

        # public_titulaire_territoriale = salarie['public_titulaire_territoriale'].combine_tax_scales()
        # public_titulaire_hospitaliere = salarie['public_titulaire_hospitaliere'].combine_tax_scales()
        # public_non_titulaire = salarie['public_non_titulaire'].combine_tax_scales()

        # Pour a fonction publique la csg est calculée sur l'ensemble salbrut(=TIB) + primes
        # Imposable = TIB - csg( (1+taux_prime)*TIB ) - pension(TIB) + taux_prime*TIB
        bareme_csg_public_titulaire_etat = csg.multiply_rates(
            1 + TAUX_DE_PRIME, inplace = False, new_name = 'csg deduc titutaire etat')
        public_titulaire_etat.add_tax_scale(bareme_csg_public_titulaire_etat)
        bareme_prime = MarginalRateTaxScale(name = 'taux de prime')
        bareme_prime.add_bracket(0, -TAUX_DE_PRIME)  # barème équivalent à taux_prime*TIB
        public_titulaire_etat.add_tax_scale(bareme_prime)

        traitement_indiciaire_brut = (
            (categorie_salarie == TypesCategorieSalarie.public_titulaire_etat)
            * public_titulaire_etat.inverse().calc(salaire_imposable_pour_inversion)
            )

        # TODO: complete this to deal with the fonctionnaire
        # supplement_familial_traitement = 0  # TODO: dépend de salbrut
        # indemnite_residence = 0  # TODO: fix bug
        return traitement_indiciaire_brut


class primes_fonction_publique(Variable):
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''Calcule les primes.
        '''
        # Get value for year and divide below.
        traitement_indiciaire_brut = individu('traitement_indiciaire_brut',
            period.start.offset('first-of', 'year').period('year'))

        return TAUX_DE_PRIME * traitement_indiciaire_brut


class inversion_directe_salaires(Reform):
    key = 'inversion_directe_salaires'
    name = 'Inversion des revenus'

    def apply(self):
        self.add_variable(salaire_imposable_pour_inversion)
        for variable in [traitement_indiciaire_brut, primes_fonction_publique, salaire_de_base]:
            self.update_variable(variable)



from numpy import datetime64

from openfisca_france.model.base import Individu, Variable, MONTH, date, TypesActivite, TypesStatutOccupationLogement, Menage


class date_entree_logement(Variable):
    value_type = date
    default_value = date(1970, 1, 1)
    entity = Menage
    definition_period = MONTH
    label = 'Date entrée dans le logement'


class mon_job_mon_logement_eligibilite_logement(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = "Eligibilité logement de l'aide mon jon mon logement"

    def formula(individu, period, parameters):
        statut_occupation_logement = individu.menage('statut_occupation_logement', period)
        params = parameters(period).prestations_sociales.aides_logement.action_logement.mon_job_mon_logement

        locataire = ((statut_occupation_logement == TypesStatutOccupationLogement.locataire_hlm)
                     + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_vide)
                     + (statut_occupation_logement == TypesStatutOccupationLogement.locataire_meuble))
        eligibilite_date_entree_logement = individu.menage('date_entree_logement', period) > datetime64(period.offset(-params.delai_max_en_mois_apres_entree_logement, 'month').start)
        return locataire * eligibilite_date_entree_logement


class mon_job_mon_logement_eligibilite_jeunes_actifs(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = "Éligibilité à l'aide mon job mon logement pour les jeunes actifs"

    def formula(individu, period, parameters):
        params = parameters(period).prestations_sociales.aides_logement.action_logement.mon_job_mon_logement.jeune_actif

        eligibilite_activite = (individu('activite', period) == TypesActivite.actif) + individu('alternant', period)
        smic_mensuel_brut = individu('smic_proratise', period)
        eligibilite_salaire = (individu('salaire_de_base', period) <= smic_mensuel_brut) * (individu('salaire_de_base', period) >= smic_mensuel_brut * params.pourcentage_minimum_smic)
        eligibilite_logement = individu('mon_job_mon_logement_eligibilite_logement', period)
        eligibilite_activite_debut = individu('contrat_de_travail_debut', period) > datetime64(period.offset(-params.delai_max_en_mois_apres_debut_contrat_de_travail, 'month').start)
        elibilite_age = individu('age', period) < params.age_maximum

        return eligibilite_activite * eligibilite_salaire * eligibilite_logement * eligibilite_activite_debut * elibilite_age


class mon_job_mon_logement_eligibilite(Variable):
    value_type = bool
    entity = Individu
    definition_period = MONTH
    label = "Éligibilité à l'aide mon job mon logement"

    def formula(individu, period, parameters):
        params = parameters(period).prestations_sociales.aides_logement.action_logement.mon_job_mon_logement
        eligibilite_activite = (individu('activite', period) == TypesActivite.actif) + individu('alternant', period)
        smic_mensuel_brut = individu('smic_proratise', period)
        eligibilite_salaire = individu('salaire_de_base', period) <= smic_mensuel_brut * params.pourcentage_maximum_smic
        eligibilite_logement = individu('mon_job_mon_logement_eligibilite_logement', period)
        eligibilite_activite_debut = individu('contrat_de_travail_debut', period) >= datetime64(
            period.offset(-params.delai_max_en_mois_apres_debut_contrat_de_travail, 'month').start)

        return eligibilite_activite * eligibilite_salaire * eligibilite_logement * eligibilite_activite_debut


class mon_job_mon_logement(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    label = "Montant de l'aide mon job mon logement"
    reference = 'https://www.actionlogement.fr/aide-mon-job-mon-logement'

    def formula(individu, period, parameters):
        montant = parameters(period).prestations_sociales.aides_logement.action_logement.mon_job_mon_logement.montant
        eligibilite = individu('mon_job_mon_logement_eligibilite', period) + individu('mon_job_mon_logement_eligibilite_jeunes_actifs', period)
        return montant * eligibilite



import os

from ..model.base import *


dir_path = os.path.join(os.path.dirname(__file__), 'parameters')


def modify_parameters(parameters):
    file_path = os.path.join(dir_path, 'trannoy_wasmer.yaml')
    reform_parameters_subtree = load_parameter_file(name = 'trannoy_wasmer', file_path = file_path)

    parameters.add_child('charge_loyer', reform_parameters_subtree)
    return parameters


class charges_deduc(Variable):
    label = 'Charge déductibles intégrant la charge pour loyer (Trannoy-Wasmer)'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        cd1 = foyer_fiscal('cd1', period)
        cd2 = foyer_fiscal('cd2', period)
        charge_loyer = foyer_fiscal('charge_loyer', period)

        return cd1 + cd2 + charge_loyer


class charge_loyer(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Charge déductible pour paiement d'un loyer"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        nbptr = foyer_fiscal('nbptr', period)

        loyer = foyer_fiscal.declarant_principal.menage('loyer', period, options = [ADD])

        charge_loyer = parameters(period).charge_loyer

        plaf = charge_loyer.plaf
        plaf_nbp = charge_loyer.plaf_nbp
        plafond = plaf * (not_(plaf_nbp) + plaf * nbptr * plaf_nbp)

        return 12 * min_(loyer / 12, plafond)


class trannoy_wasmer(Reform):
    name = 'Loyer comme charge déductible (Trannoy-Wasmer)'

    def apply(self):
        self.update_variable(charges_deduc)
        self.add_variable(charge_loyer)
        self.modify_parameters(modifier_function = modify_parameters)



from openfisca_france.model.base import Individu, Variable, ETERNITY, MONTH
from openfisca_france.model.prestations.education import TypesClasse


class sortie_academie(Variable):
    value_type = bool
    label = "Mention d'un vœu de changement d'académie entre le lycée et les études supérieures"
    entity = Individu
    definition_period = ETERNITY


class aide_mobilite_parcoursup(Variable):
    value_type = float
    label = "Montant de l'aide à la mobilité ParcourSup"
    entity = Individu
    definition_period = MONTH
    reference = 'https://www.etudiant.gouv.fr/fr/le-repas-au-crous-passe-1-euro-pour-tous-les-etudiants-2314'

    def formula(individu, period, parameters):
        sortie_academie = individu('sortie_academie', period)
        en_terminale = individu('annee_etude', period) == TypesClasse.terminale
        boursier = individu.famille('bourse_lycee', period) > 0

        montant = parameters(period).prestations_sociales.aides_jeunes.mobilite.parcoursup.montant

        return montant * sortie_academie * en_terminale * boursier


class sortie_region_academique(Variable):
    value_type = bool
    label = 'Changement de région académique entre la 3ème année de licence (L3) et la 1ère année de master (M1)'
    entity = Individu
    definition_period = ETERNITY


class aide_mobilite_master(Variable):
    '''
    Critères non pris en compte :
    - Première inscription en master l'année qui suit l'obtention de la licence
    '''
    value_type = float
    label = "Montant de l'aide à la mobilité Master"
    entity = Individu
    definition_period = MONTH
    reference = [
        "Décret n° 2017-969 du 10 mai 2017 relatif à l'aide à la mobilité accordée aux étudiants inscrits en première année du diplôme national de master",
        'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000034675851/'
        ]

    def formula(individu, period, parameters):
        sortie_academie = individu('sortie_region_academique', period)
        annee_etude = individu('annee_etude', period)
        en_transition = (annee_etude == TypesClasse.licence_3) + (annee_etude == TypesClasse.master_1)
        boursier = individu('boursier', period)

        montant = parameters(period).prestations_sociales.aides_jeunes.mobilite.master.montant

        return montant * sortie_academie * en_transition * boursier



import logging


from openfisca_core.taxscales import MarginalRateTaxScale
from openfisca_france.model.base import *


# TODO:
# Manquent:
# - les agriculteurs
# - les cotisations minimales
# - la gestion de la temporatité

# Il manque également le régime micro social qui consiste en un forfait unique couvrant l'ensemble des cotisations ainsi que la csg et la crds
# https://www.legifrance.gouv.fr/codes/section_lc/LEGITEXT000006073189/LEGISCTA000037051840/#LEGISCTA000037051840


log = logging.getLogger(__name__)


class categorie_non_salarie(Variable):
    value_type = Enum
    possible_values = TypesCategorieNonSalarie
    default_value = TypesCategorieNonSalarie.non_pertinent
    entity = Individu
    label = 'Type du travailleur salarié (artisan, commercant, profession libérale, etc)'
    definition_period = YEAR


class cotisations_non_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales des travailleurs non salaries'
    definition_period = YEAR

    def formula(individu, period, parameters):
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
        profession_liberale = (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)

        deces_artisan_commercant = individu('deces_artisan_commercant', period)
        famille_independant = individu('famille_independant', period)
        formation_artisan_commercant = individu('formation_artisan_commercant', period)
        retraite_complementaire_artisan_commercant = individu('retraite_complementaire_artisan_commercant', period)
        maladie_maternite_artisan_commercant = individu('maladie_maternite_artisan_commercant', period)
        vieillesse_artisan_commercant = individu('vieillesse_artisan_commercant', period)
        formation_profession_liberale = individu('formation_profession_liberale', period)
        maladie_maternite_profession_liberale = individu('maladie_maternite_profession_liberale', period)
        vieillesse_profession_liberale = individu('vieillesse_profession_liberale', period)
        retraite_complementaire_profession_liberale = individu('retraite_complementaire_profession_liberale', period)

        cotisations_non_salarie = (
            (artisan + commercant) * (
                deces_artisan_commercant
                + famille_independant
                + formation_artisan_commercant
                + retraite_complementaire_artisan_commercant
                + maladie_maternite_artisan_commercant
                + vieillesse_artisan_commercant
                )
            + profession_liberale * (
                famille_independant
                + formation_profession_liberale
                + maladie_maternite_profession_liberale
                + vieillesse_profession_liberale
                + retraite_complementaire_profession_liberale
                )
            )
        return cotisations_non_salarie


class deces_artisan_commercant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation décès des artisans et invalidité-décès des commercants'
    definition_period = YEAR

    def formula_2004(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        deces_ac = parameters(period).prelevements_sociaux.cotisations_taxes_independants_artisans_commercants.deces_ac
        # Artisan
        bareme_artisan = MarginalRateTaxScale(name = 'deces_artisan')
        bareme_artisan.add_bracket(0, deces_ac.artisans.sous_pss)
        bareme_artisan.add_bracket(1, 0)
        bareme_artisan.multiply_thresholds(plafond_securite_sociale_annuel)
        # Commercant (Invalidite + Deces)
        bareme_commercant = MarginalRateTaxScale(name = 'deces_commercant')
        bareme_commercant.add_bracket(0, deces_ac.commercants_industriels.sous_pss)
        bareme_commercant.add_bracket(1, 0)
        bareme_commercant.multiply_thresholds(plafond_securite_sociale_annuel)
        # Calcul du montant
        assiette = individu('rpns_imposables', period)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
        return -bareme_artisan.calc(assiette * artisan) - bareme_commercant.calc(assiette * commercant)

    def formula_1975(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        deces_ac = parameters(period).prelevements_sociaux.cotisations_taxes_independants_artisans_commercants.deces_ac
        # Avant 2004, le montant était forfaitaire pour les commerçants
        montant_commercant = deces_ac.commercants_industriels.montant_forfaitaire_total
        categorie_non_salarie = individu('categorie_non_salarie', period)
        # Artisan
        bareme_artisan = MarginalRateTaxScale(name = 'deces_artisan')
        bareme_artisan.add_bracket(0, deces_ac.artisans.sous_pss)
        bareme_artisan.add_bracket(1, 0)
        bareme_artisan.multiply_thresholds(plafond_securite_sociale_annuel)
        assiette = individu('rpns_imposables', period)
        # Type
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)

        return -bareme_artisan.calc(assiette * artisan) - (montant_commercant * commercant)


class formation_artisan_commercant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation formation des artisans et des commercants'
    definition_period = YEAR

    def formula_2015(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        formation = parameters(period).prelevements_sociaux.cotisations_taxes_independants_artisans_commercants.formation_ac
        # Artisan
        bareme_artisan = MarginalRateTaxScale(name = 'formation_artisan')
        bareme_artisan.add_bracket(0, formation.artisans.sous_pss)
        bareme_artisan.add_bracket(1, 0)
        bareme_artisan.multiply_thresholds(plafond_securite_sociale_annuel)
        # Commercant
        bareme_commercant = MarginalRateTaxScale(name = 'formation_commercant')
        bareme_commercant.add_bracket(0, formation.commercants_industriels.sous_pss)
        bareme_commercant.add_bracket(1, 0)
        bareme_commercant.multiply_thresholds(plafond_securite_sociale_annuel)
        assiette = individu('rpns_imposables', period)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
        return -bareme_artisan.calc(assiette * artisan) - bareme_commercant.calc(assiette * commercant)


class maladie_maternite_artisan_commercant_taux(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie et maternité des artisans et des commercants'
    definition_period = YEAR

    def formula_2020_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            )
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            ) * individu('rpns_imposables', period)
        assiette_pss = assiette / plafond_securite_sociale_annuel

        taux = where(assiette_pss != 0, (
            0.0085 + ((0.041 - 0.0085) * min_(max_(assiette_pss, 0), 0.4) / 0.4)
            + ((0.072 - 0.041) * min_(max_((assiette_pss) - 0.4, 0), 0.7) / (1.1 - 0.4))
            - (0.007 * (assiette_pss > 5) * ((assiette_pss - 5) / assiette_pss))
            ), 0)

        return artisan * taux

    def formula_2018_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            )
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            ) * individu('rpns_imposables', period)
        assiette_pss = assiette / plafond_securite_sociale_annuel
        taux = (
            0.0085 + ((0.041 - 0.0085) * min_(max_(assiette_pss, 0), 0.4) / 0.4)
            + ((0.072 - 0.041) * min_(max_((assiette_pss) - 0.4, 0), 0.7) / (1.1 - 0.4))
            - (0.007 * (assiette_pss > 5))
            )

        return artisan * where(assiette_pss != 0, taux, 0)

    def formula_2017_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            )
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            ) * individu('rpns_imposables', period)
        taux = (0.03 + (0.065 - 0.03) * min_(max_(assiette / plafond_securite_sociale_annuel, 0), 0.7) / 0.7) + 0.007

        return artisan * taux


class maladie_maternite_artisan_commercant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie et maternité des artisans et des commercants'
    definition_period = YEAR

    def formula(individu, period):
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            ) * individu('rpns_imposables', period)
        taux = individu('maladie_maternite_artisan_commercant_taux', period)

        return -(taux * assiette)


class retraite_complementaire_artisan_commercant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation de la retraite complémentaire des artisans et des commercants'
    definition_period = YEAR

    def formula_2013(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        retraite_complementaire = parameters(period).prelevements_sociaux.cotisations_taxes_independants_artisans_commercants.ret_comp_ac.art_ind_com
        montant_du_plafond_rci = retraite_complementaire.montant_du_plafond_rci
        bareme = MarginalRateTaxScale(name = 'retraite_complementaire')
        bareme.add_bracket(0, retraite_complementaire.sous_plafond_rci)
        bareme.add_bracket(montant_du_plafond_rci, retraite_complementaire.entre_1_plafond_rci_et_4_plafonds_pss)
        bareme.add_bracket(4 * plafond_securite_sociale_annuel, 0)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            ) * individu('rpns_imposables', period)
        return -bareme.calc(assiette)


class vieillesse_artisan_commercant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation vieillesse (plafonnée et déplafonnée) des artisans et des commercants'
    definition_period = YEAR

    def formula_2014(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        vieillesse_artisan_commercant = parameters(period).prelevements_sociaux.cotisations_taxes_independants_artisans_commercants.ret_ac
        bareme = MarginalRateTaxScale(name = 'vieillesse')
        categorie_non_salarie = individu('categorie_non_salarie', period)
        # Les taux sous_pss sont les mêmes pour artisans et commercants
        bareme.add_bracket(0, vieillesse_artisan_commercant.artisans.sous_pss + vieillesse_artisan_commercant.tous_independants.tout_salaire)
        bareme.add_bracket(1, vieillesse_artisan_commercant.tous_independants.tout_salaire)
        bareme.multiply_thresholds(plafond_securite_sociale_annuel)
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
        assiette = (artisan + commercant) * individu('rpns_imposables', period)
        return -bareme.calc(assiette)


class famille_independant(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation famille des indépendants'
    definition_period = YEAR

    def formula_2018_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            + (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            ) * individu('rpns_imposables', period)
        taux = (
            0 + (.031) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel - 1.1,
                    0
                    ),
                (1.4 - 1.1)
                )
            / (1.4 - 1.1)
            )
        return - (taux * assiette)

    def formula_2015_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
            + (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
            + (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            ) * individu('rpns_imposables', period)
        taux = (
            0.0215 + (.031) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel - 1.1,
                    0
                    ),
                (1.4 - 1.1)
                )
            / (1.4 - 1.1)
            )
        return - (taux * assiette)


class formation_profession_liberale(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation formation professionelle des professions libérales'
    definition_period = YEAR

    def formula(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        bareme = MarginalRateTaxScale(name = 'formation_profession_liberale')
        taux = parameters(period).prelevements_sociaux.professions_liberales.formation_pl.sous_pss
        bareme.add_bracket(0, taux)
        bareme.add_bracket(1, 0)
        bareme.multiply_thresholds(plafond_securite_sociale_annuel)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            ) * individu('rpns_imposables', period)
        return -bareme.calc(assiette)


class maladie_maternite_profession_liberale(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie maternité des professions libérales'
    definition_period = YEAR

    def formula_2022_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            * individu('rpns_imposables', period)
            )
        taux = (
            .015 + (.065 - .015) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel,
                    0
                    ),
                1.1
                )
            / 1.1
            )
        cotisation_supplementaire = 0.003 * min_(assiette, (3 * plafond_securite_sociale_annuel))

        return - (
            (taux * assiette) + cotisation_supplementaire
            )

    def formula_2021_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            * individu('rpns_imposables', period)
            )
        taux = (
            .015 + (.065 - .015) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel,
                    0
                    ),
                1.1
                )
            / 1.1
            )
        cotisation_supplementaire = 0.0015 * min_(assiette, (3 * plafond_securite_sociale_annuel))

        return - (
            (taux * assiette) + cotisation_supplementaire
            )

    def formula_2018_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            * individu('rpns_imposables', period)
            )
        taux = (
            .015 + (.065 - .015) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel,
                    0
                    ),
                1.1
                )
            / 1.1
            )
        return - (
            taux * assiette
            )

    def formula_2017_01_01(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            * individu('rpns_imposables', period)
            )
        taux = (
            .03 + (.065 - .03) * min_(
                max_(
                    assiette / plafond_securite_sociale_annuel,
                    0
                    ),
                0.7
                )
            / 0.7
            )
        return - (
            taux * assiette
            )

    def formula_2012_12_31(individu, period, parameters):
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            * individu('rpns_imposables', period)
            )
        return - (
            0.065 * assiette
            )


class retraite_complementaire_profession_liberale(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation de retraite complémentarie des professions libérales'
    definition_period = YEAR

    def formula_2013(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        bareme = MarginalRateTaxScale(name = 'retraite_complementaire')
        bareme.add_bracket(0, .09)  # TODO taux à la louche car hétérogène
        bareme.add_bracket(5, 0)  # TODO on peut améliorer le calcul car on a les parametres
        bareme.multiply_thresholds(plafond_securite_sociale_annuel)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            ) * individu('rpns_imposables', period)
        return -bareme.calc(assiette)


class vieillesse_profession_liberale(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation retraite des professions libérales'
    definition_period = YEAR

    def formula_2015(individu, period, parameters):
        plafond_securite_sociale_annuel = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_annuel
        bareme = MarginalRateTaxScale(name = 'vieillesse')
        assurance_vieillesse = parameters(period).prelevements_sociaux.professions_liberales.ret_pl.assurance_vieillesse
        bareme.add_bracket(0, assurance_vieillesse.sous_1_pss)
        bareme.add_bracket(1, assurance_vieillesse.entre_1_et_5_pss)
        bareme.add_bracket(5, 0)
        bareme.multiply_thresholds(plafond_securite_sociale_annuel)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        assiette = (
            (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
            ) * individu('rpns_imposables', period)
        return -bareme.calc(assiette)



from openfisca_france.model.base import *
from openfisca_core import reforms


class date_simulation(Variable):
    value_type = date
    entity = Individu
    label = 'Date de la simulation'
    definition_period = MONTH


class aah(Variable):
    calculate_output = calculate_output_add
    value_type = float
    label = 'Allocation adulte handicapé (Individu) mensualisée'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        # Si l'évaluation est nécessaire
        # utilise, si possible, le montant de la période de simulation
        #
        # Permet une meilleure évaluation du RSA lorsque l'AAH n'est pas connue.
        known_periods = individu.get_holder('date_simulation').get_known_periods()
        if known_periods:
            date_demande = sorted(known_periods, reverse = True)[0]
            return individu('aah_base', date_demande)

        return individu('aah_base', period)


class simulation_reform(reforms.Reform):

    def apply(self):
        self.add_variable(date_simulation)
        self.update_variable(aah)



from openfisca_france.model.base import *


class aeeh_niveau_handicap(Variable):
    value_type = int
    entity = Individu
    label = "Catégorie de handicap prise en compte pour l'AEEH"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class aeeh(Variable):
    value_type = float
    entity = Famille
    label = "Allocation d'éducation de l'enfant handicapé"
    reference = 'http://vosdroits.service-public.fr/particuliers/N14808.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add

    def formula_2006_01_01(famille, period, parameters):
        '''Allocation d'éducation de l'enfant handicapé.

        Rremplace l'allocation d'éducation spéciale (AES) depus le 1er janvier 2006.
        Ce montant peut être majoré par un complément accordé par la Cdaph qui prend en compte :
        le coût du handicap de l'enfant,
        la cessation ou la réduction d'activité professionnelle d'un ou l'autre des deux parents,
        l'embauche d'une tierce personne rémunérée.
        Une majoration est versée au parent isolé bénéficiaire d'un complément d'Aeeh lorsqu'il cesse ou réduit
        son activité professionnelle ou lorsqu'il embauche une tierce personne rémunérée.
        '''
        janvier = period.this_year.first_month
        isole = not_(famille('en_couple', janvier))
        prestations_familiales = parameters(period).prestations_sociales.prestations_familiales

        base = prestations_familiales.education_presence_parentale.aeeh.base
        complement_d_allocation = prestations_familiales.education_presence_parentale.aeeh.complement_allocation
        majoration = prestations_familiales.education_presence_parentale.aeeh.majoration_parent_isole

        age = famille.members('age', janvier)
        handicap = famille.members('handicap', janvier)
        niveau_handicap = famille.members('aeeh_niveau_handicap', period)
        # Indicatrice d'isolement pour les indidivus
        isole = famille.project(isole)

        enfant_handicape = handicap * (age < prestations_familiales.education_presence_parentale.aeeh.age_maximum_enfant)

        montant_par_enfant = enfant_handicape * prestations_familiales.bmaf.bmaf * (
            base
            + (niveau_handicap == 1) * complement_d_allocation._children['1ere_categorie']
            + (niveau_handicap == 2) * (complement_d_allocation._children['1ere_categorie'] + majoration._children['2e_categorie'] * isole)
            + (niveau_handicap == 3) * (complement_d_allocation._children['2e_categorie'] + majoration._children['3e_categorie'] * isole)
            + (niveau_handicap == 4) * (complement_d_allocation._children['3e_categorie'] + majoration._children['4e_categorie'] * isole)
            + (niveau_handicap == 5) * (complement_d_allocation._children['4e_categorie'] + majoration._children['5e_categorie'] * isole)
            + (niveau_handicap == 6) * majoration._children['6e_categorie'] * isole
            ) + (niveau_handicap == 6) * complement_d_allocation._children['6e_categorie']

        montant_total = famille.sum(montant_par_enfant, role = Famille.ENFANT)

        # L'attribution de l'AEEH de base et de ses compléments éventuels ne fait pas obstacle au
        # versement des prestations familiales.
        # L'allocation de présence parentale peut être cumulée avec l'AEEH de base, mais pas avec son
        # complément ni avec la majoration de parent isolé.
        # Tous les éléments de la prestattion de compensation du handicap (PCH) sont également ouverts
        # aux bénéficiaires de l'AEEH de base, sous certaines conditions, mais ce cumul est exclusif du
        # complément de l'AEEH. Les parents d'enfants handicapés doivent donc choisir entre le versement
        # du complément d'AEEH et la PCH.

        # Ces allocations ne sont pas soumises à la CRDS
        return montant_total


class aes(Variable):
    value_type = float
    entity = Famille
    label = "Allocation d'éducation spéciale"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add
    end = '2005-12-31'

    def formula_2002_04_01(famille, period, parameters):
        janvier = period.this_year.first_month
        isole = not_(famille('en_couple', janvier))
        prestations_familiales = parameters(period).prestations_sociales.prestations_familiales

        base = prestations_familiales.education_presence_parentale.aes.base
        complement_d_allocation = prestations_familiales.education_presence_parentale.aes.complement_d_allocation
        complement_d_allocation._children['6e_categorie'] = complement_d_allocation._children['6e_categorie']

        age = famille.members('age', janvier)
        handicap = famille.members('handicap', janvier)
        niveau_handicap = famille.members('aeeh_niveau_handicap', period)
        # Indicatrice d'isolement pour les indidivus
        isole = famille.project(isole)

        enfant_handicape = handicap * (age < prestations_familiales.education_presence_parentale.aes.age_maximum_enfant)

        montant_par_enfant = enfant_handicape * prestations_familiales.bmaf.bmaf * (
            base
            + (niveau_handicap == 1) * complement_d_allocation._children['1ere_categorie']
            + (niveau_handicap == 2) * complement_d_allocation._children['1ere_categorie']
            )

        montant_total = famille.sum(montant_par_enfant, role = Famille.ENFANT)
        return montant_total



from openfisca_france.model.base import *


class ada(Variable):
    value_type = float
    entity = Famille
    label = "Montant mensuel  de l'aide pour demandeur d'asile"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_11(famille, period, parameters):
        nb_parents = famille('nb_parents', period)
        af_nbenf = famille('af_nbenf', period)
        place_hebergement = famille('place_hebergement', period)
        asile_demandeur = famille('asile_demandeur', period)
        ada = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.ada

        nb_pers = af_nbenf + nb_parents
        ada_par_jour = (
            ada.montant_journalier_pour_une_personne
            + (nb_pers - 1) * ada.majoration_pers_supp
            + ada.supplement_non_hebergement
            * not_(place_hebergement)
            )

        montant_ada = period.days * ada_par_jour * asile_demandeur
        return montant_ada


class asile_demandeur(Variable):
    value_type = bool
    entity = Famille
    label = "Famille demandant l'asile"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class place_hebergement(Variable):
    value_type = bool
    default_value = True
    entity = Famille
    label = "Bénéficie d'une place dans un centre d'hébergement"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
